import typing

from .ElemType import ElemType
from .MeshEdge import MeshEdge
from .MeshElement import MeshElement
from .MeshFace import MeshFace
from .MeshNode import MeshNode
from ..BasicGeometry.Cell import Cell
from ..BasicGeometry.Edge import Edge
from ..BasicGeometry.Face import Face
from ..BasicGeometry.IgnoredVertex import IgnoredVertex
from ..Datum.DatumCsys import DatumCsys
from ..Part.PartBase import PartBase
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class MeshPart(PartBase):
    """The following commands operate on Part objects. For more information about the Part
    object, see Part object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import mesh
    """

    @abaqus_method_doc
    def assignStackDirection(self, cells: typing.Tuple[Cell, ...], referenceRegion: Face):
        """This method assigns a stack direction to geometric cells. The stack direction will be
        used to orient the elements during mesh generation.

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying regions where to assign the stack direction.
        referenceRegion
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the top side of the stack direction.
        """
        ...

    @abaqus_method_doc
    def associateMeshWithGeometry(
        self,
        geometricEntity: str,
        elements: typing.Tuple[MeshElement, ...] = (),
        elemFaces: typing.Tuple[MeshFace, ...] = (),
        elemEdges: typing.Tuple[MeshEdge, ...] = (),
        node: MeshNode = MeshNode((0, 0, 0)),
    ):
        """This method associates a geometric entity with mesh entities that are either orphan
        elements, bounds orphan elements, or were created using the bottom-up meshing technique.

        Parameters
        ----------
        geometricEntity
            A Cell, a Face, an Edge, or a ConstrainedSketchVertex object specifying geometric entity to be associated
            with one or more mesh entities.If the geometric entity is a Cell object then the
            argument **elements** must be specified.If the geometric entity is a :py:class:`~abaqus.BasicGeometry.Face.Face` object then the
            argument **elemFaces** must be specified.If the geometric entity is an Edge object then
            the argument **elemEdges** must be specified.If the geometric entity is a ConstrainedSketchVertex object
            then the argument **node** must be specified.
        elements
            A sequence of MeshElement objects specifying the elements to be associated with the
            geometric cell.
        elemFaces
            A sequence of Mesh:py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the element faces to be associated with the
            geometric face.
        elemEdges
            A sequence of MeshEdge objects specifying the element edges to be associated with the
            geometric edge.
        node
            A :py:class:`~abaqus.Mesh.MeshNode.MeshNode` object specifying the mesh node to be associated with the geometric vertex.
        """
        ...

    @abaqus_method_doc
    def createVirtualTopology(
        self,
        regions: typing.Tuple[Face, ...] = (),
        mergeShortEdges: Boolean = False,
        shortEdgeThreshold: float = None,
        mergeSmallFaces: Boolean = False,
        smallFaceAreaThreshold: float = None,
        mergeSliverFaces: Boolean = False,
        faceAspectRatioThreshold: float = None,
        mergeSmallAngleFaces: Boolean = False,
        smallFaceCornerAngleThreshold: float = None,
        mergeThinStairFaces: Boolean = False,
        thinStairFaceThreshold: float = None,
        ignoreRedundantEntities: Boolean = False,
        cornerAngleTolerance: float = 30,
        applyBlendControls: Boolean = False,
        blendSubtendedAngleTolerance: float = None,
        blendRadiusTolerance: float = None,
    ):
        """This method creates a virtual topology feature by automatically merging faces and edges
        based on a set of geometric parameters. The edges and vertices that are being merged
        will be ignored during mesh generation.

        Parameters
        ----------
        regions
            A sequence of :py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the domain to search for geometric entities that
            need to be merged. Entities identified as candidates to be merged may be merged with
            entities from outside the specified region. If **regions** is not specified then the
            entire part is the domain for searching geometric entities that need to be merged.
        mergeShortEdges
            A Boolean specifying whether to merge short edges. The default value is False.
        shortEdgeThreshold
            A Float specifying a threshold that determines which edges are considered to be short.
            These edges are the candidate entities to be merged. This argument is a required
            argument if the argument*mergeShortEdges* equals True and it is ignored if the argument
            **mergeShortEdges** equals False.
        mergeSmallFaces
            A Boolean specifying whether to merge faces with small area. The default value is False.
        smallFaceAreaThreshold
            A Float specifying a threshold that determines which faces are considered to have a
            small area. These faces are the candidate entities to be merged. This argument is a
            required argument if the argument*mergeSmallFaces* equals True and it is ignored if the
            argument **mergeSmallFaces** equals False.
        mergeSliverFaces
            A Boolean specifying whether to merge faces with high aspect ratio. The default value is
            False.
        faceAspectRatioThreshold
            A Float specifying a threshold that determines which faces are considered to have high
            aspect ratio. These faces are the candidate entities to be merged. This argument is a
            required argument if the argument*mergeSliverFaces* equals True and it is ignored if the
            argument **mergeSliverFaces** equals False.
        mergeSmallAngleFaces
            A Boolean specifying whether to merge faces that have a sharp corner angle. The default
            value is False.
        smallFaceCornerAngleThreshold
            A Float specifying a threshold that determines which face corner angles are considered
            to be small. These faces will be candidate entities to be merged. This argument is a
            required argument if the argument*mergeSmallAngleFaces* equals True and it is ignored if
            the argument **mergeSmallAngleFaces** equals False.
        mergeThinStairFaces
            A Boolean specifying whether to merge faces that represent a thin stair-like feature.
            The default value is False.
        thinStairFaceThreshold
            A Float specifying a threshold that determines which faces representing small stair-like
            features are considered thin. These faces will be candidate entities to be merged. This
            argument is required if the argument **mergeThinStairFaces** is True and it is ignored if
            **mergeThinStairFaces** is False.
        ignoreRedundantEntities
            A Boolean specifying whether to abstract away redundant edges and vertices. The default
            value is False.
        cornerAngleTolerance
            A Float specifying the angle deviation from 180 degrees at a vertex or at an edge such
            that the two edges radiating from the vertex or the two faces bounded by the edge can be
            merged. The default value is 30.0 degrees.
        applyBlendControls
            A Boolean specifying whether to verify that blend faces can be merged with neighboring
            faces. If **applyBlendControls** is true then all faces that have angle larger than
            **blendSubtendedAngleTolerance** and a radius smaller than **blendRadiusTolerance** will not
            be merged with neighboring faces unless the neighboring faces are also blend faces with
            similar geometric characteristics. The default value is False.
        blendSubtendedAngleTolerance
            A Float specifying the largest subtended angle of blend faces that can be merged with
            neighboring faces. This argument is a required argument if the argument
            **applyBlendControls** equals True and it is ignored if the argument **applyBlendControls**
            equals False.
        blendRadiusTolerance
            A Float specifying the smallest radius of curvature of blend faces that can be merged
            with neighboring faces. This argument is a required argument if the argument
            **applyBlendControls** equals True and it is ignored if the argument **applyBlendControls**
            equals False.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Assembly.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def deleteBoundaryLayerControls(self, regions: typing.Tuple[Cell, ...]):
        """This method deletes the control parameters for boundary layer mesh for all the specified
        regions.

        Parameters
        ----------
        regions
            A sequence of Cell objects specifying the regions for which to set the boundary layer
            mesh control parameters.
        """
        ...

    @abaqus_method_doc
    def deleteMesh(self, regions: typing.Tuple["Part"]):
        """This method deletes a subset of the mesh that contains the native elements from the
        given parts or regions.

        Parameters
        ----------
        regions
            A sequence of Part objects or Region objects specifying the parts or regions from which
            the native mesh is to be deleted.
        """
        ...

    @abaqus_method_doc
    def deleteMeshAssociationWithGeometry(
        self, geometricEntities: typing.Tuple[Cell, ...], addBoundingEntities: Boolean = False
    ):
        """This method deletes the association of geometric entities with mesh entities.

        Parameters
        ----------
        geometricEntities
            A sequence of Cell objects, :py:class:`~abaqus.BasicGeometry.Face.Face` objects, Edge objects, or ConstrainedSketchVertex objects specifying the
            geometric entities that will be disassociated from the mesh.
        addBoundingEntities
            A Boolean specifying whether the mesh will also be disassociated from the geometric
            entities that bounds the given **geometricEntities**. For example, if the argument
            **geometricEntities** contains a face, this boolean indicates whether the edges and
            vertices that bound the face will also be disassociated from the mesh. The default value
            is False.
        """
        ...

    @abaqus_method_doc
    def deletePreviewMesh(self):
        """This method deletes all boundary meshes in the parts. See the **boundaryPreview** argument
        of generateMesh for information about generating boundary meshes.
        """
        ...

    @abaqus_method_doc
    def deleteSeeds(self, regions: typing.Tuple["Part"]):
        """This method deletes the global edge seeds from the given parts or deletes the local edge
        seeds from the given edges.

        Parameters
        ----------
        regions
            A sequence of Part objects or Edge objects specifying the parts or edges from which the
            seeds are to be deleted.
        """
        ...

    @abaqus_method_doc
    def generateMesh(
        self,
        regions: typing.Tuple["Part"] = (),
        seedConstraintOverride: Boolean = OFF,
        meshTechniqueOverride: Boolean = OFF,
        boundaryPreview: Boolean = OFF,
        boundaryMeshOverride: Boolean = OFF,
    ):
        """This method generates a mesh in the given parts or regions.

        Parameters
        ----------
        regions
            A sequence of Part objects or Region objects specifying the parts or regions where the
            mesh is to be generated.
        seedConstraintOverride
            A Boolean specifying whether mesh generation is allowed to modify seed constraints. The
            default value is OFF.
        meshTechniqueOverride
            A Boolean specifying whether mesh generation is allowed to modify the existing mesh
            techniques so that a compatible mesh can be generated. The default value is OFF.
        boundaryPreview
            A Boolean specifying whether the generated mesh should be a boundary preview mesh. This
            option will only have an effect if any of the specified regions are to be meshed with
            tetrahedral elements or using the bottom-up technique with hexahedral or wedge elements.
            The default value is OFF.
        boundaryMeshOverride
            A Boolean specifying whether mesh generation is allowed to modify an existing boundary
            preview mesh. This option will only have an effect if any of the specified regions are
            to be meshed with tetrahedral elements and a boundary preview mesh already exists. The
            default value is OFF.
        """
        ...

    @abaqus_method_doc
    def generateBottomUpExtrudedMesh(
        self,
        cell: Cell,
        numberOfLayers: int,
        extrudeVector: tuple,
        geometrySourceSide: str = "",
        elemFacesSourceSide: typing.Tuple[MeshFace, ...] = (),
        elemSourceSide: tuple = (),
        depth: float = None,
        targetSide: str = "",
        biasRatio: float = 1,
        extendElementSets: Boolean = False,
    ):
        """This method generates solid elements by extruding a 2D mesh along a vector, either on an
        orphan mesh or within a cell region using a bottom-up technique.

        Parameters
        ----------
        cell
            A :py:class:`~abaqus.BasicGeometry.Cell.Cell` object specifying the geometric region where the mesh is to be generated. This
            argument is valid only for native parts.
        numberOfLayers
            An Int specifying the number of layers to be generated along the extrusion vector.
        extrudeVector
            A sequence of sequences of Floats specifying the start point and end point of a vector.
            Each point is defined by a tuple of three coordinates indicating its position. The
            direction of the mesh extrusion operation is from the first point to the second point.
        geometrySourceSide
            A Region of :py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the geometric domain to be used as the source for
            the extrude meshing operation.
        elemFacesSourceSide
            A sequence of Mesh:py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the faces of 3D elements to be used as the
            source for the extrude meshing operation.
        elemSourceSide
            A sequence of 2D MeshElement objects specifying the elements to be used as the source
            for the extrude meshing operation.
        depth
            A Float specifying the distance of the mesh extrusion. If unspecified, the vector length
            of the **extrudeVector** argument is assumed.
        targetSide
            A datum plane, a sequence of :py:class:`~abaqus.BasicGeometry.Face.Face` objects, a sequence of Mesh:py:class:`~abaqus.BasicGeometry.Face.Face` objects, or a sequence
            of 2D MeshElement objects specifying the target of the extrude meshing operation. If
            specified, this argument overrides the **depth** argument, and all points on the source
            will be extruded in the direction of the extrusion vector until meeting the target.
        biasRatio
            A Float specifying a ratio of the element size in the extrusion direction between the
            source and the target sides of the extrusion. The default is 1.0, meaning no bias.
        extendElementSets
            A Boolean specifying whether existing element sets that include source elements will be
            extended to also include extruded elements. This argument is ignored for native parts.
            The default value is False.
        """
        ...

    @abaqus_method_doc
    def generateBottomUpSweptMesh(
        self,
        cell: Cell,
        geometrySourceSide: str = "",
        elemFacesSourceSide: typing.Tuple[MeshFace, ...] = (),
        elemSourceSide: tuple = (),
        geometryConnectingSides: str = "",
        elemFacesConnectingSides: typing.Tuple[MeshFace, ...] = (),
        elemConnectingSides: tuple = (),
        targetSide: Face = None, 
        numberOfLayers: int = None,
        extendElementSets: Boolean = False,
    ):
        """This method generates solid elements by sweeping a 2D mesh, either on an orphan mesh or
        within a cell region using a bottom-up technique.

        Parameters
        ----------
        cell
            A :py:class:`~abaqus.BasicGeometry.Cell.Cell` object specifying the geometric region where the mesh is to be generated. This
            argument is valid only for native parts.
        geometrySourceSide
            A Region of :py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the geometric domain to be used as the source for
            the sweep meshing operation.
        elemFacesSourceSide
            A sequence of Mesh:py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the faces of 3D elements to be used as the
            source for the sweep meshing operation.
        elemSourceSide
            A sequence of 2D MeshElement objects specifying the elements to be used as the source
            for the sweep meshing operation.
        geometryConnectingSides
            A Region of :py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying connecting sides of the sweep meshing operation.
        elemFacesConnectingSides
            A sequence of Mesh:py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying connecting sides of the sweep meshing
            operation.
        elemConnectingSides
            A sequence of 2D MeshElement objects specifying connecting sides of the sweep meshing
            operation.
        targetSide
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the target side of the sweep meshing operation.
        numberOfLayers
            An Int specifying the number of layers to be generated along the sweep direction.
        extendElementSets
            A Boolean specifying whether existing element sets that include source elements will be
            extended to also include swept elements. This argument is ignored for native parts. The
            default value is False.
        """
        ...

    @abaqus_method_doc
    def generateBottomUpRevolvedMesh(
        self,
        cell: Cell,
        numberOfLayers: int,
        axisOfRevolution: tuple,
        angleOfRevolution: float,
        geometrySourceSide: str = "",
        elemFacesSourceSide: typing.Tuple[MeshFace, ...] = (),
        elemSourceSide: tuple = (),
        extendElementSets: Boolean = False,
    ):
        """This method generates solid elements by revolving a 2D mesh around an axis, either on an
        orphan mesh or within a cell region using a bottom-up technique.

        Parameters
        ----------
        cell
            A :py:class:`~abaqus.BasicGeometry.Cell.Cell` object specifying the geometric region where the mesh is to be generated. This
            argument is valid only for native parts.
        numberOfLayers
            An Int specifying the number of layers of elements to be generated around the axis of
            revolution.
        axisOfRevolution
            A sequence of sequences of Floats specifying the two points of the vector that describes
            the axis of revolution. Each point is defined by a tuple of three coordinates indicating
            its position. The direction of the axis of revolution is from the first point to the
            second point. The orientation of the revolution operation follows the right-hand-rule
            about the axis of revolution.
        angleOfRevolution
            A Float specifying the angle of revolution.
        geometrySourceSide
            A Region of :py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the geometric domain to be used as the source for
            the revolve meshing operation.
        elemFacesSourceSide
            A sequence of Mesh:py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the faces of 3D elements to be used as the
            source for the revolve meshing operation.
        elemSourceSide
            A sequence of 2D MeshElement objects specifying the elements to be used as the source
            for the revolve meshing operation.
        extendElementSets
            A Boolean specifying whether existing element sets that include source elements will be
            extended to also include extruded elements. This argument is ignored for native parts.
            The default value is False.
        """
        ...

    @abaqus_method_doc
    def getEdgeSeeds(
        self, edge: Edge, attribute: typing.Union[SymbolicConstant, float]
    ):
        """This method returns an edge seed parameter for a specified edge of a part.

        Parameters
        ----------
        edge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the edge to be queried.
        attribute
            A SymbolicConstant specifying the type of edge seed attribute to return. Possible values
            are:

            - EDGE_SEEDING_METHOD
            - BIAS_METHOD
            - NUMBER
            - AVERAGE_SIZE
            - DEVIATION_FACTOR
            - MIN_SIZE_FACTOR
            - BIAS_RATIO
            - BIAS_MIN_SIZE
            - BIAS_MAX_SIZE
            - VERTEX_ADJ_TO_SMALLEST_ELEM
            - SMALLEST_ELEM_LOCATION
            - CONSTRAINT

        Returns
        -------
        typing.Union[float, int, SymbolicConstant]
            The return value is a Float, an Int, or a SymbolicConstant depending on the value of the
            **attribute** argument.

            The return value is dependent on the **attribute** argument.

            - If **attribute** = EDGE_SEEDING_METHOD, the return value is a SymbolicConstant specifying
            the edge seeding method used to create the seeds along the edge. Possible values are: UNIFORM_BY_NUMBER, UNIFORM_BY_SIZE, CURVATURE_BASED_BY_SIZE, BIASED, NONE

            - If **attribute** = BIAS_METHOD, the return value is a SymbolicConstant specifying the bias
            type used to create the seeds along the edge. Possible values are: SINGLE, DOUBLE, NONE

            - If **attribute** = NUMBER, the return value is an Int specifying the number of element
            seeds along the edge.
            - If **attribute** = AVERAGE_SIZE, the return value is a Float specifying the average
            element size along the edge.
            - If **attribute** = DEVIATION_FACTOR, the return value is a Float specifying the deviation
            factor h/Lh/L, where hh is the chordal deviation and LL is the element length. If edge
            seeds are not defined, the return value is zero.
            - If **attribute** = MIN_SIZE_FACTOR, the return value is a Float specifying the size of the
            smallest allowable element as a fraction of the specified global element size. If edge
            seeds are not defined, the return value is zero.
            - If **attribute** = BIAS_RATIO, the return value is a Float specifying the length ratio of
            the largest element to the smallest element.
            - If **attribute** = BIAS_MIN_SIZE, the return value is a Float specifying the length of the
            largest element; only applicable if the EDGE_SEEDING_METHOD is BIASED and seeds were
            specified by minimum and maximum sizes.
            - If **attribute** = BIAS_MAX_SIZE, the return value is a Float specifying the length of the
            largest element; only applicable if the EDGE_SEEDING_METHOD is BIASED and seeds were
            specified by minimum and maximum sizes.
            - If **attribute** = VERTEX_ADJ_TO_SMALLEST_ELEM, the return value is an Int specifying the
            ID of the vertex next to the smallest element; only applicable if the
            EDGE_SEEDING_METHOD is BIASED.
            - If **attribute** = SMALLEST_ELEM_LOCATION, the return value is a SymbolicConstant
            specifying the location of smallest elements for double bias seeds; only applicable if
            the EDGE_SEEDING_METHOD is BIASED and BIAS_METHOD is DOUBLE. Possible values are: SMALLEST_ELEM_AT_CENTER, SMALLEST_ELEM_AT_ENDS, NONE

            - If **attribute** = CONSTRAINT, the return value is a SymbolicConstant specifying how close
            the seeds must be matched by the mesh. Possible values are: FREE, FINER, FIXED, NONE

            A value of NONE indicates that the edge is not seeded.
        """
        ...

    @abaqus_method_doc
    def getElementType(self, region: str, elemShape: SymbolicConstant):
        """This method returns the ElemType object of a given element shape assigned to a region of
        a part.

        Parameters
        ----------
        region
            A Cell, a Face, or an Edge object specifying the region to be queried.
        elemShape
            A SymbolicConstant specifying the shape of the element for which to return the element
            type. Possible values are:

            - LINE
            - QUAD
            - TRI
            - HEX
            - WEDGE
            - TET

        Returns
        -------
        ElementType
            An ElemType object.

        Raises
        ------
        TypeError
            The region cannot be associated with element types or the **elemShape** is not
            consistent with the dimension of the **region**.
        """
        ...

    @abaqus_method_doc
    def getIncompatibleMeshInterfaces(self, cells: typing.Tuple[Cell, ...] = ()):
        """This method returns a sequence of :py:class:`~abaqus.BasicGeometry.Face.Face` objects that are meshed with incompatible
        elements.

        Parameters
        ----------
        cells
            A sequence of cell objects which will be used to search the incompatible faces.

        Returns
        -------
        typing.Tuple[Face, ...]
            A sequence of :py:class:`~abaqus.BasicGeometry.Face.Face` objects.
        """
        ...

    @abaqus_method_doc
    def getMeshControl(self, region: str, attribute: SymbolicConstant):
        """This method returns a mesh control parameter for the specified region of a part.

        Parameters
        ----------
        region
            A Cell, a Face, or an Edge object specifying the region to be queried.
        attribute
            A SymbolicConstant specifying the mesh control attribute to return. Possible values are:

            - ELEM_SHAPE
            - TECHNIQUE
            - ALGORITHM
            - MIN_TRANSITION

            The return value is dependent on the **attribute** argument.
            - If **attribute** = ELEM_SHAPE, the return value is a SymbolicConstant specifying the
              element shape used during meshing. Possible values are: LINE, QUAD, TRI,  QUAD_DOMINATED, HEX, TET, WEDGE, HEX_DOMINATED

            - If **attribute** = TECHNIQUE, the return value is a SymbolicConstant specifying the
              meshing technique to be used during meshing. Possible values are: FREE, STRUCTURED, SWEEP, UNMESHABLE, Where UNMESHABLE indicates that no meshing technique is applicable with the currently assigned element shape.
            - If **attribute** = ALGORITHM, the return value is a SymbolicConstant specifying the
              meshing algorithm to be used during meshing. Possible values are: MEDIAL_AXIS, ADVANCING_FRONT, DEFAULT, NON_DEFAULT, NONE, Where NONE indicates that no algorithm is applicable.
            - If **attribute** = MIN_TRANSITION, the return value is a Boolean indicating whether
              minimum transition will be used during meshing. This option is applicable only to the
              following: Free quadrilateral meshing or sweep hexahedral meshing with **algorithm** = MEDIAL_AXIS, Structured quadrilateral meshing.

        Returns
        -------
        typing.Union[bool, SymbolicConstant]
            The return value is a SymbolicConstant or a Boolean depending on the value of the
            **attribute** argument.

        Raises
        ------
        TypeError
            The region cannot carry mesh controls.
        """
        ...

    @abaqus_method_doc
    def getMeshStats(self, regions: tuple):
        """This method returns the mesh statistics for the given regions.

        Parameters
        ----------
        regions
            A sequence or tuple of ConstrainedSketchGeometry regions for which mesh statistics should be returned.

        Returns
        -------
        MeshStats
            A :py:class:`~abaqus.Mesh.MeshStats.MeshStats` object.
        """
        ...

    @abaqus_method_doc
    def getPartSeeds(self, attribute: typing.Union[SymbolicConstant, float]):
        """This method returns a part seed parameter for the part.

        Parameters
        ----------
        attribute
            A SymbolicConstant specifying the type of part seed attribute to return. Possible values
            are:
            
            - SIZE
            - DEFAULT_SIZE
            - DEVIATION_FACTOR
            - MIN_SIZE_FACTOR
            
            The return value depends on the value of the **attribute** argument.
            
            - If **attribute** = SIZE, the return value is a Float specifying the assigned global
              element size. If part seeds are not defined, the return value is zero.
            - If **attribute** = DEFAULT_SIZE, the return value is a Float specifying a suggested
              default global element size based upon the part geometry.
            - If **attribute** = DEVIATION_FACTOR, the return value is a Float specifying the deviation
              factor h/Lh/L, where hh is the chordal deviation and LL is the element length. If part
              seeds are not defined, the return value is zero.
            - If **attribute** = MIN_SIZE_FACTOR, the return value is a Float specifying the size of the
              smallest allowable element as a fraction of the specified global element size. If part
              seeds are not defined, the return value is zero.

        Returns
        -------
        float
            The return value is a Float that depends on the value of the **attribute** argument.

        Raises
        ------
        Error: Part does not contain native geometry            
            An exception occurs if the part does not contain native geometry.
        """
        ...

    @abaqus_method_doc
    def getUnmeshedRegions(self):
        """This method returns all geometric regions in the part that require a mesh for submitting
        an analysis but are either unmeshed or are meshed incompletely.

        Returns
        -------
        Region
            A :py:class:`~abaqus.Region.Region.Region` object, or None.
        """
        ...

    @abaqus_method_doc
    def ignoreEntity(self, entities: tuple):
        """This method creates a virtual topology feature. Virtual topology allows unimportant
        entities to be ignored during mesh generation. You can combine two adjacent faces by
        specifying a common edge to ignore. Similarly, you can combine two adjacent edges by
        specifying a common vertex to ignore.

        Parameters
        ----------
        entities
            A sequence of vertices and edges specifying the entities to be ignored during meshing.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Assembly.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def restoreIgnoredEntity(self, entities: typing.Tuple[IgnoredVertex, ...]):
        """This method restores vertices and edges that have been merged using a virtual topology
        feature.

        Parameters
        ----------
        entities
            A sequence of IgnoredVertex objects and IgnoredEdge objects specifying the entities to
            be restored.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Assembly.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def seedEdgeByBias(
        self,
        biasMethod: SymbolicConstant,
        end1Edges: typing.Tuple[Edge, ...],
        end2Edges: typing.Tuple[Edge, ...],
        centerEdges: typing.Tuple[Edge, ...],
        endEdges: typing.Tuple[Edge, ...],
        ratio: float,
        number: int,
        minSize: float,
        maxSize: float,
        constraint: SymbolicConstant = FREE,
    ):
        """This method seeds the given edges nonuniformly using the specified number of elements
        and bias ratio or the specified minimum and maximum element sizes.

        Parameters
        ----------
        biasMethod
            A SymbolicConstant specifying whether single- or double-biased seed distribution will be
            applied. If unspecified, single-biased seed distribution will be applied. Possible
            values are:
            - SINGLE: Single-biased seed distribution will be applied.
            - DOUBLE: Double-biased seed distribution will be applied.
        end1Edges
            A sequence of Edge objects specifying the edges to seed. The smallest elements will be
            positioned near the end where the normalized curve parameter=0.0. You must provide
            either the **end1Edges** or the **end2Edges** argument or both when **biasMethod** = SINGLE and
            omit both of them when **biasMethod** = DOUBLE.Note:You can determine which end is which by
            the order of the vertex indices returned by
            [getVertices()](https://help.3ds.com/2022/english/DSSIMULIA_Established/SIMACAEKERRefMap/simaker-c-edgepyc.htm?ContextScope=all#simaker-edgegetverticespyc).
        end2Edges
            A sequence of Edge objects specifying the edges to seed. The smallest elements will be
            positioned near the end where the normalized curve parameter=1.0.
        centerEdges
            A sequence of Edge objects specifying the edges to seed. The smallest elements will be
            positioned near edge center. You must provide either the **centerEdges** or the **endEdges**
            argument or both when **biasMethod** = DOUBLE and omit both of them when
            **biasMethod** = SINGLE.
        endEdges
            A sequence of Edge objects specifying the edges to seed. The smallest elements will be
            positioned near edge ends.
        ratio
            A Float specifying the ratio of the largest element to the smallest element. Possible
            values are 1.0 ≤≤ **ratio** ≤≤ 106.
        number
            An Int specifying the number of elements along each edge. Possible values are 1 ≤≤
            **number** ≤≤ 104.
        minSize
            A Float specifying the desired smallest element size.
        maxSize
            A Float specifying the desired largest element size.Note:You must specify either the
            **ratio** and **number** or **minSize** and **maxSize** pair of arguments.
        constraint
            A SymbolicConstant specifying how closely the seeds must be matched by the mesh. The
            default value is FREE. If unspecified, the existing constraint will remain unchanged.
            Possible values are:

            - FREE: The resulting mesh can be finer or coarser than the specified seeds.
            - FINER: The resulting mesh can be finer than the specified seeds.
            - FIXED: The seeds must be exactly matched by the mesh (only with respect to the number
              of elements, not to the nodal positioning).
        """
        ...

    @abaqus_method_doc
    def seedEdgeByNumber(
        self, edges: typing.Tuple[Edge, ...], number: int, constraint: SymbolicConstant = FREE
    ):
        """This method seeds the given edges uniformly based on the number of elements along the
        edges.

        Parameters
        ----------
        edges
            A sequence of Edge objects specifying the edges to seed.
        number
            An Int specifying the number of elements along each edge. Possible values are 1 ≤≤
            **number** ≤≤ 104.
        constraint
            A SymbolicConstant specifying how closely the seeds must be matched by the mesh. The
            default value is FREE. If unspecified, the existing constraint will remain unchanged.
            Possible values are:

            - FREE: The resulting mesh can be finer or coarser than the specified seeds.
            - FINER: The resulting mesh can be finer than the specified seeds.
            - FIXED: The seeds must be exactly matched by the mesh (only with respect to the number
              of elements, not to the nodal positioning).
        """
        ...

    @abaqus_method_doc
    def seedEdgeBySize(
        self,
        edges: typing.Tuple[Edge, ...],
        size: float,
        deviationFactor: float = None,
        minSizeFactor: float = None,
        constraint: SymbolicConstant = FREE,
    ):
        """This method seeds the given edges either uniformly or following edge curvature
        distribution, based on the desired element size.

        Parameters
        ----------
        edges
            A sequence of Edge objects specifying the edges to seed.
        size
            A Float specifying the desired element size.
        deviationFactor
            A Float specifying the deviation factor h/Lh/L, where hh is the chordal deviation and LL
            is the element length.
        minSizeFactor
            A Float specifying the size of the smallest allowable element as a fraction of the
            specified global element size.
        constraint
            A SymbolicConstant specifying how closely the seeds must be matched by the mesh. The
            default value is FREE. If unspecified, the existing constraint will remain unchanged.
            Possible values are:

            - FREE: The resulting mesh can be finer or coarser than the specified seeds.
            - FINER: The resulting mesh can be finer than the specified seeds.
            - FIXED: The seeds must be exactly matched by the mesh (only with respect to the number
              of elements, not to the nodal positioning).
        """
        ...

    @abaqus_method_doc
    def seedPart(
        self,
        size: float,
        deviationFactor: float = None,
        minSizeFactor: float = None,
        constraint: SymbolicConstant = FREE,
    ):
        """This method assigns global edge seeds to the given parts.

        Parameters
        ----------
        size
            A Float specifying the desired global element size for the edges.
        deviationFactor
            A Float specifying the deviation factor h/Lh/L, where hh is the chordal deviation and LL
            is the element length.
        minSizeFactor
            A Float specifying the size of the smallest allowable element as a fraction of the
            specified global element size.
        constraint
            A SymbolicConstant specifying how closely the seeds must be matched by the mesh. The
            default value is FREE. If unspecified, the existing constraint will remain unchanged.
            Possible values are:FREE: The resulting mesh can be finer or coarser than the specified
            seeds.FINER: The resulting mesh can be finer than the specified seeds.
        """
        ...

    @abaqus_method_doc
    def setBoundaryLayerControls(
        self,
        regions: typing.Tuple[Cell, ...],
        firstElemSize: float,
        growthFactor: float,
        numLayers: int,
        inactiveFaces: typing.Tuple[Face, ...] = (),
        setName: str = "",
    ):
        """This method sets the control parameters for boundary layer mesh for the specified
        regions.

        Parameters
        ----------
        regions
            A sequence of Cell objects specifying the regions for which to set the boundary layer
            mesh control parameters.
        firstElemSize
            A Float specifying the height of the first element layer off boundary. Possible values
            are 0.0 << **firstElemSize** ≤≤ 106.
        growthFactor
            A Float specifying the ratio of heights of any two consecutive element layers. Possible
            values are 1.0 ≤≤ **growthFactor** ≤≤ 10.0.
        numLayers
            An Int specifying the number of element layers to be generated. Possible values are 1 ≤≤
            **numLayers** ≤≤ 104.
        inactiveFaces
            A sequence of :py:class:`~abaqus.BasicGeometry.Face.Face` objects specifying the faces where boundary layer should not be
            generated. By default, boundary layer mesh will be generated on all faces of the
            selected regions.
        setName
            A String specifying a unique name for a set that will contain boundary layer elements.
        """
        ...

    @abaqus_method_doc
    def setElementType(self, regions: tuple, elemTypes: typing.Tuple[ElemType, ...]):
        """This method assigns element types to the specified regions.

        Parameters
        ----------
        regions
            A sequence of ConstrainedSketchGeometry regions or MeshElement objects, or a Set object containing either
            geometry regions or elements, specifying the regions to which element types are to be
            assigned.
        elemTypes
            A sequence of ElemType objects, one for each element shape applicable to the
            regions.Note:If an ElemType object has an UNKNOWN_*xxx* value for **elemCode**, its order
            will be deduced from the order of other valid ElemType objects within the same
            setElementType command. If no valid ElemType objects can be found, the order will remain
            unchanged.

        Raises
        ------
        Exception
            As a result of the element assignment, a region must have the same library, family, and
            order for all its assigned element types. Otherwise, an exception will be thrown.
            For example, suppose the Hex, Wedge, and Tet elements previously assigned to a cell are
            all linear. The user now constructs an ElemType object with a quadratic Hex element and
            includes only this object in the setElementType command. An exception will be thrown
            because the Wedge and Tet elements will remain linear (i.e., As Is) and become
            incompatible with the newly assigned quadratic Hex element.
        """
        ...

    @abaqus_method_doc
    def setLogicalCorners(self, region: str, corners: str):
        """This method sets the logical corners for a mappable face region.

        Parameters
        ----------
        region
            A Face region.
        corners
            Three, four, or five ConstrainedSketchVertex objects defining the logical corners for a given mappable
            face region.
        """
        ...

    @abaqus_method_doc
    def setMeshControls(
        self,
        regions: tuple,
        elemShape: SymbolicConstant = None,
        technique: SymbolicConstant = None,
        algorithm: SymbolicConstant = None,
        minTransition: Boolean = ON,
        sizeGrowth: SymbolicConstant = None,
        allowMapped: Boolean = OFF,
    ):
        """This method sets the mesh control parameters for the specified regions.

        Parameters
        ----------
        regions
            A sequence of Face or Cell regions specifying the regions for which to set the mesh
            control parameters.
        elemShape
            A SymbolicConstant specifying the element shape to be used for meshing. The default
            value is QUAD for Face regions and HEX for Cell regions. If unspecified, the existing
            element shape will remain unchanged. Possible values are:

            - QUAD: Quadrilateral mesh.
            - QUAD_DOMINATED: Quadrilateral-dominated mesh.
            - TRI: Triangular mesh.
            - HEX: Hexahedral mesh.
            - HEX_DOMINATED: Hex-dominated mesh.
            - TET: Tetrahedral mesh.
            - WEDGE: Wedge mesh.
        technique
            A SymbolicConstant specifying the mesh technique to be used. The default value is FREE
            for Face regions. For Cell regions the initial value depends on the geometry of the
            regions and can be STRUCTURED, SWEEP, or unmeshable. If unspecified, the existing mesh
            technique(s) will remain unchanged. Possible values are:

            - FREE: Free mesh technique.
            - STRUCTURED: Structured mesh technique.
            - SWEEP: Sweep mesh technique.
            - BOTTOM_UP: Bottom-up mesh technique. Only applicable for cell regions.
            - SYSTEM_ASSIGN: Allow the system to assign a suitable technique. The actual technique
              assigned can be STRUCTURED, SWEEP, or unmeshable.
        algorithm
            A SymbolicConstant specifying the algorithm used to generate the mesh for the specified
            regions. Possible values are MEDIAL_AXIS, ADVANCING_FRONT, and NON_DEFAULT. If
            unspecified, the existing value will remain unchanged. This option is applicable only to
            the following:
            
            - Free quadrilateral or quadrilateral-dominated meshing. In this case the possible
              values are MEDIAL_AXIS and ADVANCING_FRONT.
            - Sweep hexahedral or hexahedral-dominated meshing. In this case the possible values are
              MEDIAL_AXIS and ADVANCING_FRONT.
            - Free tetrahedral meshing. In this case the only possible value is NON_DEFAULT, and it
              indicates that the free tetrahedral-meshing technique available in Abaqus 6.4 or earlier
              will be used. If algorithm is not specified, the default
        minTransition
            A Boolean specifying whether minimum transition is to be applied. The default value is
            ON. If unspecified, the existing value will remain unchanged. This option is applicable
            only in the following cases:
            
            - Free quadrilateral meshing or hexahedral sweep meshing with **algorithm** = MEDIAL_AXIS.
            - Structured quadrilateral meshing.
        sizeGrowth
            A SymbolicConstant specifying element size growth to be applied when generating the
            interior of a tetrahedral mesh. Possible values are MODERATE and MAXIMUM. If
            unspecified, the existing value will remain unchanged. This option only applies to the
            default tetrahedral mesher.
        allowMapped
            A Boolean specifying whether mapped meshing can be used to replace the selected mesh
            technique. The **allowMapped** argument is applicable only in the following cases:
            
            - Free triangular meshing.
            - Free quadrilateral or quadrilateral-dominated meshing with
              **algorithm** = ADVANCING_FRONT.
            - Hexahedral or hexahedral-dominated sweep meshing with **algorithm** = ADVANCING_FRONT.
            - Free tetrahedral meshing. **allowMapped** = True implies that mapped triangular meshing
              can be used on faces that bound three-dimensional **regions**.
        """
        ...

    @abaqus_method_doc
    def setSweepPath(self, region: str, edge: Edge, sense: SymbolicConstant):
        """This method sets the sweep path for a sweepable region or the revolve path for a
        revolvable region.

        Parameters
        ----------
        region
            A sweepable region.
        edge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the sweep or revolve path.
        sense
            A SymbolicConstant specifying the sweep sense. The sense will affect only how gasket
            elements will be created; it will have no effect if gasket elements are not used.
            Possible values are FORWARD or REVERSE.If **sense** = FORWARD, the sense of the given edge's
            underlying curve will be used.
        """
        ...

    @abaqus_method_doc
    def verifyMeshQuality(
        self,
        criterion: SymbolicConstant,
        threshold: float = None,
        elemShape: SymbolicConstant = None,
        regions: tuple = (),
    ):
        """This method tests the mesh quality of a part and returns poor-quality elements.

        Parameters
        ----------
        criterion
            A SymbolicConstant specifying the criterion used for the quality check. Possible values
            are:
            
            - ANALYSIS_CHECKS
              When this criterion is specified Abaqus/CAE will invoke the element quality checks
              included with the input file processor for Abaqus/Standard and Abaqus/Explicit.
            - ANGULAR_DEVIATION
              The maximum amount (in degrees) that an element's face corner angles deviate from the
              ideal angle. The ideal angle is 90° for quadrilateral element faces and 60° for
              triangular element faces. Elements with an angular deviation larger than the specified
              threshold will fail this test.
            - ASPECT_RATIO
              The ratio between the lengths of the longest and shortest edges of an element. Elements
              with an aspect ratio larger than the specified threshold will fail this test.
            - GEOM_DEVIATION_FACTOR
              The largest geometric deviation factor evaluated along any of the element edges
              associated with geometric edges or faces. The geometric deviation factor along an
              element edge is calculated by dividing the maximum gap between the element edge and its
              associated geometry by the length of the element edge. Elements with a geometric
              deviation factor larger than the specified threshold will fail this test.
            - LARGE_ANGLE
              The largest corner angle on any of an element's faces. Elements with face angles larger
              than the specified threshold (in degrees) will fail this test.
            - LONGEST_EDGE
              The length of an element's longest edge. Elements with an edge longer than the specified
              threshold will fail this test.
            - MAX_FREQUENCY
              An estimate of an element's contribution to the initial maximum allowable frequency for
              Abaqus/Standard analyses. This calculation requires appropriate section assignments and
              material definitions. Elements whose maximum allowable frequency is smaller than the
              given value will fail this test.
            - SHAPE_FACTOR
              The shape factor for triangular and tetrahedral elements. This is the ratio between the
              element area or volume and the optimal element area or volume. Elements with a shape
              factor smaller than the specified threshold will fail this test.
            - SHORTEST_EDGE
              The length of an element's shortest edge. Elements with an edge shorter than the
              specified threshold will fail this test.
            - SMALL_ANGLE
              The smallest corner angle on any of an element's faces. Elements with face angles
              smaller than the given value (in degrees) will fail this test.
            - STABLE_TIME_INCREMENT
              An estimate of an element's contribution to the initial maximum stable time increment
              for Abaqus/Explicit analyses. This calculation requires appropriate section assignments
              and material definitions. Elements that require a time increment smaller than the given
              value will fail this test.
        threshold
            A Float value used to determine low quality elements according to the specified
            criterion. This argument is ignored when the ANALYSIS_CHECKS criterion is used. For
            other criterion, if this argument is unspecified then no list of failed elements will be
            returned.
        elemShape
            A SymbolicConstant specifying an element shape for limiting the query. Possible values
            are LINE, QUAD, TRI, HEX, WEDGE, and TET.
        regions
            A sequence of Region or MeshElement objects. If you do not specify the **regions**
            argument, the entire part mesh is considered.

        Returns
        -------
        typing.Dict[str, int | float, MeshElement]
            A Dictionary object containing values for some number of the following keys:
            failedElements, warningElements, naElements (sequences of MeshElement objects);
            numElements (Int); average, worst (Float); worstElement 
            (:py:class:`~abaqus.Mesh.MeshElement.MeshElement` object) .
        """
        ...

    @abaqus_method_doc
    def Node(
        self, coordinates: tuple, localCsys: DatumCsys = None,  label: int = None
    ):
        """This method creates a node on an orphan mesh part.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].parts[name].Node

        Parameters
        ----------
        coordinates
            A sequence of three Floats specifying the coordinates of the new node.
        localCsys
            A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system. If unspecified, the global
            coordinate system will be used.
        label
            An Int specifying the node label.

        Returns
        -------
        MeshNode
            A :py:class:`~abaqus.Mesh.MeshNode.MeshNode` object.
        """
        node = MeshNode(coordinates, localCsys, label)
        self.nodes.append(node)
        return node



from ..PlotOptions.DGCommonOptions import DGCommonOptions
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class CommonOptions(DGCommonOptions):
    """The CommonOptions object stores values and attributes that are common to all plot
    states. The CommonOptions object has no constructor command. Abaqus creates a
    *defaultOdbDisplay.commonOptions* member when you import the Visualization module.
    Abaqus creates a **commonOptions** member when it creates the OdbDisplay object, using the
    values from *defaultOdbDisplay.commonOptions*. Abaqus creates the **odbDisplay** member
    when a viewport is created, using the values from **defaultOdbDisplay**.
    CommonOptions objects are accessed in one of two ways:
    - The default common options. These settings are used as defaults when other
    **commonOptions** members are created. These settings can be set to customize user
    preferences.
    - The common options associated with a particular viewport.
    The CommonOptions object is derived from the DGCommonOptions object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import visualization
            session.defaultOdbDisplay.commonOptions
            session.viewports[name].assemblyDisplay.displayGroupInstances[name].odbDisplayOptions.commonOptions
            session.viewports[name].layers[name].assemblyDisplay.displayGroupInstances[name].odbDisplayOptions.commonOptions
            session.viewports[name].layers[name].odbDisplay.commonOptions
            session.viewports[name].layers[name].odbDisplay.displayGroupInstances[name].odbDisplayOptions.commonOptions
            session.viewports[name].layers[name].partDisplay.displayGroupInstances[name].odbDisplayOptions.commonOptions
            session.viewports[name].odbDisplay.commonOptions
            session.viewports[name].odbDisplay.displayGroupInstances[name].odbDisplayOptions.commonOptions
            session.viewports[name].partDisplay.displayGroupInstances[name].odbDisplayOptions.commonOptions
    """

    #: A SymbolicConstant specifying the deformation scale factor mode. Possible values are
    #: AUTO, UNIFORM, and NONUNIFORM. The default value is AUTO.
    deformationScaling: SymbolicConstant = AUTO

    #: A Float specifying the uniform deformation scaling constant when
    #: **deformationScaling** = UNIFORM. The default value is **autoDeformationScaleValue**.
    uniformScaleFactor: float = None

    #: A Float specifying the deformation scale factor value when **deformationScaling** = AUTO.
    #: This value is read-only.
    autoDeformationScaleValue: float = None

    #: A tuple of three Floats specifying the deformation scaling in each of the three
    #: coordinate directions when **deformationScaling** = NONUNIFORM. The default value is
    #: (*autoDeformationScaleValue*, **autoDeformationScaleValue**, **autoDeformationScaleValue**).
    nonuniformScaleFactor: float = None

    #: A SymbolicConstant specifying the render style of the plot. Possible values are
    #: WIREFRAME, FILLED, HIDDEN, and SHADED. The default value is SHADED.
    renderStyle: SymbolicConstant = SHADED

    #: A SymbolicConstant specifying which edges to plot. Possible values are ALL, EXTERIOR,
    #: FEATURE, FREE, and NONE. The default value is EXTERIOR.NONE can be used only when
    #: **renderStyle** = SHADED.
    visibleEdges: SymbolicConstant = EXTERIOR

    #: A SymbolicConstant specifying the edge line style. Possible values are SOLID, DASHED,
    #: DOTTED, and DOT_DASH. The default value is SOLID.
    edgeLineStyle: SymbolicConstant = SOLID

    #: A SymbolicConstant specifying the edge line thickness. Possible values are VERY_THIN,
    #: THIN, MEDIUM, and THICK. The default value is VERY_THIN.
    edgeLineThickness: SymbolicConstant = VERY_THIN

    #: A Boolean specifying whether to allow color coded items in the output database to
    #: override the edge and fill color settings. The default value is ON.
    colorCodeOverride: Boolean = ON

    #: A Boolean specifying whether to plot the element labels. The default value is OFF.
    elemLabels: Boolean = OFF

    #: A Boolean specifying whether to plot the face labels. The default value is OFF.
    faceLabels: Boolean = OFF

    #: A Boolean specifying whether to plot the node labels. The default value is OFF.
    nodeLabels: Boolean = OFF

    #: A Boolean specifying whether to plot the node symbols. The default value is OFF.
    nodeSymbols: Boolean = OFF

    #: A SymbolicConstant specifying the node symbol types. Possible values are:
    #: 
    #: - FILLED_CIRCLE
    #: - FILLED_SQUARE
    #: - FILLED_DIAMOND
    #: - FILLED_TRI
    #: - HOLLOW_CIRCLE
    #: - HOLLOW_SQUARE
    #: - HOLLOW_DIAMOND
    #: - HOLLOW_TRI
    #: - CROSS
    #: - XMARKER
    #: The default value is HOLLOW_CIRCLE.
    nodeSymbolType: SymbolicConstant = HOLLOW_CIRCLE

    #: A SymbolicConstant specifying the node symbol size. Possible values are SMALL, MEDIUM,
    #: and LARGE. The default value is SMALL.
    nodeSymbolSize: SymbolicConstant = SMALL

    #: A Boolean specifying whether elements are displayed in a shrunk format. The default
    #: value is OFF.
    elementShrink: Boolean = OFF

    #: An Int specifying the percentage to shrink the elements when **elementShrink** = ON.
    #: Possible values are 0≤≤ **elementShrinkPercentage** ≤≤ 90. The default value is 5.
    elementShrinkFactor: int = 5

    #: A Boolean specifying whether to scale coordinates. The default value is OFF.
    coordinateScale: Boolean = OFF

    #: A Boolean specifying whether to draw arrows that indicate the directions of element and
    #: surface normals. The default value is OFF.
    normals: Boolean = OFF

    #: A SymbolicConstant specifying whether to draw element normals or surface normals.
    #: Possible values are ELEMENT and SURFACE. The default value is ELEMENT.
    normalDisplay: SymbolicConstant = ELEMENT

    #: A SymbolicConstant specifying the length of the normal arrows. Possible values are
    #: SHORT, MEDIUM, and LONG. The default value is MEDIUM.
    normalArrowLength: SymbolicConstant = MEDIUM

    #: A SymbolicConstant specifying the thickness of the normal arrows. Possible values are
    #: VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.
    normalLineThickness: SymbolicConstant = VERY_THIN

    #: A SymbolicConstant specifying the arrowhead style of the normal arrows. Possible values
    #: are NONE, FILLED, and WIRE. The default value is WIRE.
    normalArrowheadStyle: SymbolicConstant = WIRE

    #: A Boolean specifying whether to set translucency. The default value is OFF.
    translucency: Boolean = OFF

    #: A Float specifying the translucency factor when **translucency** = ON. Possible values are
    #: 0.0≤≤ **translucencyFactor** ≤≤ 1.0. The default value is 0.3.
    translucencyFactor: float = 0

    #: A String specifying the color to be used to plot the edges of the model when
    #: **renderStyle** = WIREFRAME or HIDDEN. The default value is "White".
    edgeColorWireHide: str = ""

    #: A String specifying the color to be used to plot the edges of the model when
    #: **renderStyle** = FILLED or SHADED. The default value is "Black".
    edgeColorFillShade: str = ""

    #: A String specifying the color to be used to fill elements when **renderStyle** = FILLED or
    #: SHADED. The default value is "White".
    fillColor: str = ""

    #: A String specifying the label font to be used for all model labels. The default value is
    #: "-*-courier-medium-r-normal-*-*-120-*-*-m-*-*-*".
    labelFont: str = ""

    #: A String specifying the color to be used to plot the element labels. The default value
    #: is "Cyan".
    elemLabelColor: str = ""

    #: A String specifying the color to be used to plot the face labels. The default value is
    #: "Red".
    faceLabelColor: str = ""

    #: A String specifying the color to be used to plot the node labels. The default value is
    #: "Yellow".
    nodeLabelColor: str = ""

    #: A String specifying the color to be used to plot the node symbols. The default value is
    #: "Yellow".
    nodeSymbolColor: str = ""

    #: A String specifying the color to be used to plot the normal to a nonbeam element or to a
    #: surface. The default value is "Red".
    faceNormalColor: str = ""

    #: A String specifying the color to be used to plot an arrow along the beam n1n1-direction.
    #: The default value is "Blue".
    beamN1Color: str = ""

    #: A String specifying the color to be used to plot an arrow along the beam n2n2-direction.
    #: The default value is "Red".
    beamN2Color: str = ""

    #: A String specifying the color to be used to plot an arrow along the tangent to a beam.
    #: The default value is "White".
    beamTangentColor: str = ""

    #: A tuple of three Floats specifying the coordinate scaling in each of the three
    #: coordinate directions when **coordinateScale** = ON. The default value is (1, 1, 1).
    coordinateScaleFactors: float = None

    @abaqus_method_doc
    def setValues(
        self,
        options: "CommonOptions" = None,
        renderStyle: SymbolicConstant = SHADED,
        visibleEdges: SymbolicConstant = EXTERIOR,
        deformationScaling: SymbolicConstant = AUTO,
        uniformScaleFactor: float = None,
        nonuniformScaleFactor: tuple = (),
        edgeColorWireHide: str = "",
        edgeColorFillShade: str = "",
        edgeLineStyle: SymbolicConstant = SOLID,
        edgeLineThickness: SymbolicConstant = VERY_THIN,
        fillColor: str = "",
        colorCodeOverride: Boolean = ON,
        labelFont: str = "",
        elemLabels: Boolean = OFF,
        elemLabelColor: str = "",
        faceLabels: Boolean = OFF,
        faceLabelColor: str = "",
        nodeLabels: Boolean = OFF,
        nodeLabelColor: str = "",
        nodeSymbols: Boolean = OFF,
        nodeSymbolType: SymbolicConstant = HOLLOW_CIRCLE,
        nodeSymbolColor: str = "",
        nodeSymbolSize: SymbolicConstant = SMALL,
        elementShrink: Boolean = OFF,
        elementShrinkFactor: int = 5,
        coordinateScale: Boolean = OFF,
        coordinateScaleFactors: tuple = (),
        normals: Boolean = OFF,
        normalDisplay: SymbolicConstant = ELEMENT,
        faceNormalColor: str = "",
        beamN1Color: str = "",
        beamN2Color: str = "",
        beamTangentColor: str = "",
        normalArrowLength: SymbolicConstant = MEDIUM,
        normalLineThickness: SymbolicConstant = VERY_THIN,
        normalArrowheadStyle: SymbolicConstant = WIRE,
        translucency: Boolean = OFF,
        translucencyFactor: float = 0,
    ):
        """This method modifies the CommonOptions object.

        Parameters
        ----------
        options
            A :py:class:`~abaqus.OdbDisplay.CommonOptions.CommonOptions` object from which values are to be copied. If other arguments are also
            supplied to setValues, they will override the values in **options**. The default value is
            None.
        renderStyle
            A SymbolicConstant specifying the render style of the plot. Possible values are
            WIREFRAME, FILLED, HIDDEN, and SHADED. The default value is SHADED.
        visibleEdges
            A SymbolicConstant specifying which edges to plot. Possible values are ALL, EXTERIOR,
            FEATURE, FREE, and NONE. The default value is EXTERIOR.NONE can be used only when
            **renderStyle** = SHADED.
        deformationScaling
            A SymbolicConstant specifying the deformation scale factor mode. Possible values are
            AUTO, UNIFORM, and NONUNIFORM. The default value is AUTO.
        uniformScaleFactor
            A Float specifying the uniform deformation scaling constant when
            **deformationScaling** = UNIFORM. The default value is **autoDeformationScaleValue**.
        nonuniformScaleFactor
            A sequence of three Floats specifying the deformation scaling in each of the three
            coordinate directions when **deformationScaling** = NONUNIFORM. The default value is
            (*autoDeformationScaleValue*, **autoDeformationScaleValue**, **autoDeformationScaleValue**).
        edgeColorWireHide
            A String specifying the color to be used to plot the edges of the model when
            **renderStyle** = WIREFRAME or HIDDEN. The default value is "White".
        edgeColorFillShade
            A String specifying the color to be used to plot the edges of the model when
            **renderStyle** = FILLED or SHADED. The default value is "Black".
        edgeLineStyle
            A SymbolicConstant specifying the edge line style. Possible values are SOLID, DASHED,
            DOTTED, and DOT_DASH. The default value is SOLID.
        edgeLineThickness
            A SymbolicConstant specifying the edge line thickness. Possible values are VERY_THIN,
            THIN, MEDIUM, and THICK. The default value is VERY_THIN.
        fillColor
            A String specifying the color to be used to fill elements when **renderStyle** = FILLED or
            SHADED. The default value is "White".
        colorCodeOverride
            A Boolean specifying whether to allow color coded items in the output database to
            override the edge and fill color settings. The default value is ON.
        labelFont
            A String specifying the label font to be used for all model labels. The default value is
            "-*-courier-medium-r-normal-*-*-120-*-*-m-*-*-*".
        elemLabels
            A Boolean specifying whether to plot the element labels. The default value is OFF.
        elemLabelColor
            A String specifying the color to be used to plot the element labels. The default value
            is "Cyan".
        faceLabels
            A Boolean specifying whether to plot the face labels. The default value is OFF.
        faceLabelColor
            A String specifying the color to be used to plot the face labels. The default value is
            "Red".
        nodeLabels
            A Boolean specifying whether to plot the node labels. The default value is OFF.
        nodeLabelColor
            A String specifying the color to be used to plot the node labels. The default value is
            "Yellow".
        nodeSymbols
            A Boolean specifying whether to plot the node symbols. The default value is OFF.
        nodeSymbolType
            A SymbolicConstant specifying the node symbol types. Possible values are:

            - FILLED_CIRCLE
            - FILLED_SQUARE
            - FILLED_DIAMOND
            - FILLED_TRI
            - HOLLOW_CIRCLE
            - HOLLOW_SQUARE
            - HOLLOW_DIAMOND
            - HOLLOW_TRI
            - CROSS
            - XMARKER
            
            The default value is HOLLOW_CIRCLE.
        nodeSymbolColor
            A String specifying the color to be used to plot the node symbols. The default value is
            "Yellow".
        nodeSymbolSize
            A SymbolicConstant specifying the node symbol size. Possible values are SMALL, MEDIUM,
            and LARGE. The default value is SMALL.
        elementShrink
            A Boolean specifying whether elements are displayed in a shrunk format. The default
            value is OFF.
        elementShrinkFactor
            An Int specifying the percentage to shrink the elements when **elementShrink** = ON.
            Possible values are 0≤≤ **elementShrinkPercentage** ≤≤ 90. The default value is 5.
        coordinateScale
            A Boolean specifying whether to scale coordinates. The default value is OFF.
        coordinateScaleFactors
            A sequence of three Floats specifying the coordinate scaling in each of the three
            coordinate directions when **coordinateScale** = ON. The default value is (1, 1, 1).
        normals
            A Boolean specifying whether to draw arrows that indicate the directions of element and
            surface normals. The default value is OFF.
        normalDisplay
            A SymbolicConstant specifying whether to draw element normals or surface normals.
            Possible values are ELEMENT and SURFACE. The default value is ELEMENT.
        faceNormalColor
            A String specifying the color to be used to plot the normal to a nonbeam element or to a
            surface. The default value is "Red".
        beamN1Color
            A String specifying the color to be used to plot an arrow along the beam n1n1-direction.
            The default value is "Blue".
        beamN2Color
            A String specifying the color to be used to plot an arrow along the beam n2n2-direction.
            The default value is "Red".
        beamTangentColor
            A String specifying the color to be used to plot an arrow along the tangent to a beam.
            The default value is "White".
        normalArrowLength
            A SymbolicConstant specifying the length of the normal arrows. Possible values are
            SHORT, MEDIUM, and LONG. The default value is MEDIUM.
        normalLineThickness
            A SymbolicConstant specifying the thickness of the normal arrows. Possible values are
            VERY_THIN, THIN, MEDIUM, and THICK. The default value is VERY_THIN.
        normalArrowheadStyle
            A SymbolicConstant specifying the arrowhead style of the normal arrows. Possible values
            are NONE, FILLED, and WIRE. The default value is WIRE.
        translucency
            A Boolean specifying whether to set translucency. The default value is OFF.
        translucencyFactor
            A Float specifying the translucency factor when **translucency** = ON. Possible values are
            0.0≤≤ **translucencyFactor** ≤≤ 1.0. The default value is 0.3.

        Raises
        ------
        RangeError
        """
        ...



from .Section import Section
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class CohesiveSection(Section):
    """The CohesiveSection object defines the properties of a cohesive section.
    The CohesiveSection object is derived from the Section object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]

        The corresponding analysis keywords are:

        - COHESIVE SECTION
    """

    #: A String specifying the repository key.
    name: str

    #: A SymbolicConstant specifying the geometric assumption that defines the constitutive
    #: behavior of the cohesive elements. Possible values are TRACTION_SEPARATION, CONTINUUM,
    #: and GASKET.
    response: SymbolicConstant

    #: A String specifying the name of the material.
    material: str

    #: A SymbolicConstant specifying the method used to compute the initial thickness. Possible
    #: values are:SOLVER_DEFAULT, specifying that Abaqus will use the analysis product
    #: defaultGEOMETRY, specifying that Abaqus will compute the thickness from the nodal
    #: coordinates of the elements.SPECIFY, specifying that Abaqus will use the value given for
    #: **initialThickness**The default value is SOLVER_DEFAULT.
    initialThicknessType: SymbolicConstant = SOLVER_DEFAULT

    #: A Float specifying the initial thickness for the section. The **initialThickness**
    #: argument applies only when **initialThicknessType** = SPECIFY. The default value is 1.0.
    initialThickness: float = 1

    #: None or a Float specifying the out-of-plane thickness for the section. The default value
    #: is None.
    outOfPlaneThickness: float = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        response: SymbolicConstant,
        material: str,
        initialThicknessType: SymbolicConstant = SOLVER_DEFAULT,
        initialThickness: float = 1,
        outOfPlaneThickness: float = None,
    ):
        """This method creates a CohesiveSection object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].CohesiveSection
                session.odbs[name].CohesiveSection

        Parameters
        ----------
        name
            A String specifying the repository key.
        response
            A SymbolicConstant specifying the geometric assumption that defines the constitutive
            behavior of the cohesive elements. Possible values are TRACTION_SEPARATION, CONTINUUM,
            and GASKET.
        material
            A String specifying the name of the material.
        initialThicknessType
            A SymbolicConstant specifying the method used to compute the initial thickness. Possible
            values are:SOLVER_DEFAULT, specifying that Abaqus will use the analysis product
            defaultGEOMETRY, specifying that Abaqus will compute the thickness from the nodal
            coordinates of the elements.SPECIFY, specifying that Abaqus will use the value given for
            **initialThickness**The default value is SOLVER_DEFAULT.
        initialThickness
            A Float specifying the initial thickness for the section. The **initialThickness**
            argument applies only when **initialThicknessType** = SPECIFY. The default value is 1.0.
        outOfPlaneThickness
            None or a Float specifying the out-of-plane thickness for the section. The default value
            is None.

        Returns
        -------
        CohesiveSection
            A :py:class:`~abaqus.Section.CohesiveSection.CohesiveSection` object.

        Raises
        ------
        RangeError and InvalidNameError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        initialThicknessType: SymbolicConstant = SOLVER_DEFAULT,
        initialThickness: float = 1,
        outOfPlaneThickness: float = None,
    ):
        """This method modifies the CohesiveSection object.

        Parameters
        ----------
        initialThicknessType
            A SymbolicConstant specifying the method used to compute the initial thickness. Possible
            values are:SOLVER_DEFAULT, specifying that Abaqus will use the analysis product
            defaultGEOMETRY, specifying that Abaqus will compute the thickness from the nodal
            coordinates of the elements.SPECIFY, specifying that Abaqus will use the value given for
            **initialThickness**The default value is SOLVER_DEFAULT.
        initialThickness
            A Float specifying the initial thickness for the section. The **initialThickness**
            argument applies only when **initialThicknessType** = SPECIFY. The default value is 1.0.
        outOfPlaneThickness
            None or a Float specifying the out-of-plane thickness for the section. The default value
            is None.

        Raises
        ------
        RangeError
        """
        ...



from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc


@abaqus_class_doc
class BodyCurrentState(LoadState):
    """The BodyCurrentState object stores the propagating data of a body current in a step. One
    instance of this object is created internally by the BodyCurrent object for each step.
    The instance is also deleted internally by the BodyCurrent object.
    The BodyCurrentState object has no constructor or methods.
    The BodyCurrentState object is derived from the LoadState object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - DECURRENT
    """

    #: A Float specifying the load magnitude.
    magnitude: float = None

    #: A SymbolicConstant specifying the propagation state of the load magnitude. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    magnitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: SymbolicConstant = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from .CommandRegister import CommandRegister
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class RegisteredList(CommandRegister, list):
    """This class allows you to create a list that can be queried from the GUI and is capable
    of notifying the GUI when the contents of the list change.
    The RegisteredList object is derived from the CommandRegister object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import customKernel
    """

    @abaqus_method_doc
    def __init__(self):
        """This method creates a RegisteredList object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                customKernel.RegisteredList

        Returns
        -------
        RegisteredList
            A :py:class:`~abaqus.CustomKernel.RegisteredList.RegisteredList` object.
        """
        super().__init__()

    @abaqus_method_doc
    def Methods(self):
        """The RegisteredList object supports the same methods as a standard Python list object."""
        # TODO: Implement this method
        ...



import typing

from .AdaptiveMeshConstraint import AdaptiveMeshConstraint
from .AdaptiveMeshControl import AdaptiveMeshControl
from .DisplacementAdaptiveMeshConstraint import DisplacementAdaptiveMeshConstraint
from .RemeshingRule import RemeshingRule
from .VelocityAdaptiveMeshConstraint import VelocityAdaptiveMeshConstraint
from ..Datum.DatumCsys import DatumCsys
from ..Model.ModelBase import ModelBase
from ..Odb.Odb import Odb
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class AdaptivityModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            mdb.models[name]
    """

    @abaqus_method_doc
    def adaptiveRemesh(self, odb: Odb):
        """This method remeshes the model using the active remesh rules in the model and the error
        indicator results from a previous analysis.

        Parameters
        ----------
        odb
            An :py:class:`~abaqus.Odb.Odb.Odb` object containing error output field results.

        Returns
        -------
        AdaptivityIteration
            An :py:class:`~abaqus.Adaptivity.AdaptivityIteration.AdaptivityIteration` object.
        """
        ...

    @abaqus_method_doc
    def AdaptiveMeshConstraint(
        self,
        name: str,
        category: SymbolicConstant,
        region: Region,
        localCsys: DatumCsys = None,
    ) -> AdaptiveMeshConstraint:
        """The AdaptiveMeshConstraint object is the abstract base type for other Arbitrary
        Lagrangian Eularian (ALE) style AdaptiveMeshConstraint objects. The
        AdaptiveMeshConstraint object has no explicit constructor. The methods and members of
        the AdaptiveMeshConstraint object are common to all objects derived from the
        AdaptiveMeshConstraint object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].AdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the adaptive mesh constraint repository key.
        category
            A SymbolicConstant specifying the category of the adaptive mesh constraint. Possible values are
            MECHANICAL and THERMAL.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the adaptive mesh constraint is applied.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the adaptive mesh  constraint's
            degrees of freedom. If **localCsys** = None, the degrees of freedom are defined  in the global coordinate
            system. The default value is None.
            
        Returns
        -------
        AdaptiveMeshConstraint
            An :py:class:`~abaqus.Adaptivity.AdaptiveMeshConstraint.AdaptiveMeshConstraint` object
        """
        self.adaptiveMeshConstraints[
            name
        ] = adaptiveMeshConstraint = AdaptiveMeshConstraint(
            name, category, region, localCsys
        )
        return adaptiveMeshConstraint

    @abaqus_method_doc
    def AdaptiveMeshControl(
        self,
        name: str,
        remapping: SymbolicConstant = SECOND_ORDER_ADVECTION,
        smoothingAlgorithm: SymbolicConstant = GEOMETRY_ENHANCED,
        smoothingPriority: SymbolicConstant = UNIFORM,
        initialFeatureAngle: float = 30,
        transitionFeatureAngle: float = 30,
        momentumAdvection: SymbolicConstant = ELEMENT_CENTER_PROJECTION,
        meshingPredictor: SymbolicConstant = CURRENT,
        curvatureRefinement: float = 1,
        volumetricSmoothingWeight: float = 1,
        laplacianSmoothingWeight: float = 0,
        equipotentialSmoothingWeight: float = 0,
        meshConstraintAngle: float = 60,
        originalConfigurationProjectionWeight: float = 1,
        standardVolumetricSmoothingWeight: float = 0,
    ) -> AdaptiveMeshControl:
        """This method creates an AdaptiveMeshControl object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].AdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the name of the object.
        remapping
            A SymbolicConstant specifying the remapping algorithm. Possible values are
            FIRST_ORDER_ADVECTION and SECOND_ORDER_ADVECTION. The default value is
            SECOND_ORDER_ADVECTION.
        smoothingAlgorithm
            A SymbolicConstant specifying the type of smoothing algorithm to use. Possible values
            are STANDARD and GEOMETRY_ENHANCED. The default value is GEOMETRY_ENHANCED.
        smoothingPriority
            A SymbolicConstant specifying the type of smoothing to perform. Possible values are
            UNIFORM and GRADED. The default value is UNIFORM.
        initialFeatureAngle
            A Float specifying the initial geometric feature angle, θI, in degrees. Possible values
            are 0° ≤θI≤ 180°. The default value is 30.0.
        transitionFeatureAngle
            A Float specifying the transitional feature angle, θT, in degrees. Possible values are
            0° ≤θT≤ 180°. The default value is 30.0.
        momentumAdvection
            A SymbolicConstant specifying the type of momentum advection algorithm. Possible values
            are ELEMENT_CENTER_PROJECTION and HALF_INDEX_SHIFT. The default value is
            ELEMENT_CENTER_PROJECTION.
        meshingPredictor
            A SymbolicConstant specifying the nodal starting location to use for remeshing. Possible
            values are CURRENT and PREVIOUS. The default value is CURRENT.
        curvatureRefinement
            A Float specifying the solution dependence weight, αC. Possible values are 0.0 ≤αC≤ 1.0.
            The default value is 1.0.
        volumetricSmoothingWeight
            A Float specifying the weight used by Abaqus/Explicit for the volumetric smoothing
            method. The default value is 1.0.
        laplacianSmoothingWeight
            A Float specifying the weight for the Laplacian smoothing method. The default value is
            0.0.
        equipotentialSmoothingWeight
            A Float specifying the weight for the equipotential smoothing method. The default value
            is 0.0.
        meshConstraintAngle
            A Float specifying the initial geometric feature angle, θC. Possible values are 0° ≤θC≤
            180°. The default value is 60.0.
        originalConfigurationProjectionWeight
            A Float specifying the weight for the original configuration projection method. The
            default value is 1.0.
        standardVolumetricSmoothingWeight
            A Float specifying the weight used by Abaqus/Standard for the volumetric smoothing
            method. The default value is 0.0.

        Returns
        -------
        AdaptiveMeshControl
            An :py:class:`~abaqus.Adaptivity.AdaptiveMeshControl.AdaptiveMeshControl` object
        """
        self.adaptiveMeshControls[name] = adaptiveMeshControl = AdaptiveMeshControl(
            name,
            remapping,
            smoothingAlgorithm,
            smoothingPriority,
            initialFeatureAngle,
            transitionFeatureAngle,
            momentumAdvection,
            meshingPredictor,
            curvatureRefinement,
            volumetricSmoothingWeight,
            laplacianSmoothingWeight,
            equipotentialSmoothingWeight,
            meshConstraintAngle,
            originalConfigurationProjectionWeight,
            standardVolumetricSmoothingWeight,
        )
        return adaptiveMeshControl

    @abaqus_method_doc
    def DisplacementAdaptiveMeshConstraint(
        self,
        name: str,
        createStepName: str,
        region: Region,
        u1: typing.Union[SymbolicConstant, float] = UNSET,
        u2: typing.Union[SymbolicConstant, float] = UNSET,
        u3: typing.Union[SymbolicConstant, float] = UNSET,
        ur1: typing.Union[SymbolicConstant, float] = UNSET,
        ur2: typing.Union[SymbolicConstant, float] = UNSET,
        ur3: typing.Union[SymbolicConstant, float] = UNSET,
        amplitude: str = UNSET,
        motionType: SymbolicConstant = INDEPENDENT,
        localCsys: str = None,
    ) -> DisplacementAdaptiveMeshConstraint:
        """This method creates a DisplacementAdaptiveMeshConstraint object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].AdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the adaptive mesh constraint repository key.
        createStepName
            A String specifying the name of the step in which the adaptive mesh constraint is
            created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the adaptive mesh constraint is applied.
        u1
            A Float or a SymbolicConstant specifying the displacement component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET. Note: Although **u1**, **u2**, **u3**, **ur1**, **ur2**, and **ur3** are optional arguments, at
            least one of them must be specified.
        u2
            A Float or a SymbolicConstant specifying the displacement component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        u3
            A Float or a SymbolicConstant specifying the displacement component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        ur1
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur2
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur3
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the adaptive mesh constraint has no amplitude reference. The
            default value is UNSET. You should provide the **amplitude** argument only if it is valid
            for the specified step.
        motionType
            A SymbolicConstant specifying the mesh motion in relation to the underlying material.
            Possible values are INDEPENDENT, FOLLOW and USER_DEFINED. The default value is
            INDEPENDENT.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the adaptive mesh
            constraint's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        DisplacementAdaptiveMeshConstraint
            A :py:class:`~abaqus.Adaptivity.DisplacementAdaptiveMeshConstraint.DisplacementAdaptiveMeshConstraint` object
        """
        self.adaptiveMeshConstraints[
            name
        ] = adaptiveMeshConstraint = DisplacementAdaptiveMeshConstraint(
            name,
            createStepName,
            region,
            u1,
            u2,
            u3,
            ur1,
            ur2,
            ur3,
            amplitude,
            motionType,
            localCsys,
        )
        return adaptiveMeshConstraint

    @abaqus_method_doc
    def RemeshingRule(
        self,
        name: str,
        stepName: str,
        variables: tuple,
        description: str = "",
        region: SymbolicConstant = MODEL,
        sizingMethod: SymbolicConstant = DEFAULT,
        errorTarget: float = 0,
        maxSolutionErrorTarget: float = 0,
        minSolutionErrorTarget: float = 0,
        meshBias: int = 0,
        minElementSize: float = 0,
        maxElementSize: float = 0,
        outputFrequency: SymbolicConstant = LAST_INCREMENT,
        specifyMinSize: Boolean = OFF,
        specifyMaxSize: Boolean = ON,
        coarseningFactor: SymbolicConstant = DEFAULT_LIMIT,
        refinementFactor: SymbolicConstant = DEFAULT_LIMIT,
        elementCountLimit: int = None,
    ) -> RemeshingRule:
        """This method creates a RemeshingRule object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].AdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the name of the object.
        stepName
            A String specifying the name of the step in which resizing should occur for this rule.
        variables
            A sequence of Strings specifying the output request variables that Abaqus will use as
            error indicators.
        description
            A String specifying a descriptive string for this rule. The default value is an empty
            string.
        region
            The SymbolicConstant MODEL or a Region object specifying the region in which Abaqus will
            remesh and generate output. The SymbolicConstant MODEL represents the entire applicable
            model. The default value is MODEL.
        sizingMethod
            A SymbolicConstant specifying the method for calculating the new mesh sizes. The
            SymbolicConstant DEFAULT indicates that Abaqus will use the default calculation method
            for each individual variable. Possible values are DEFAULT, UNIFORM_ERROR, and
            MINIMUM_MAXIMUM. The default value is DEFAULT.
        errorTarget
            A Float specifying the target error percentage for each variable in the region. A value
            of 0.0 indicates that Abaqus will use automated error target reduction for the region.
            You use the **errorTarget** argument when **sizingMethod** = UNIFORM_ERROR. The default value
            is 0.0.
        maxSolutionErrorTarget
            A Float specifying the target error percentage at the location of the maximum solution
            value in the region. A value of 0.0 indicates that Abaqus will use automated error
            target reduction for the region. You use the **maxSolutionErrorTarget** argument when
            **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        minSolutionErrorTarget
            A Float specifying the target error percentage at the location of the minimum solution
            value in the region. A value of 0.0 indicates that Abaqus will use automated error
            target reduction for the region. You use the **minSolutionErrorTarget** argument when
            **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        meshBias
            An Int specifying an indication of how much Abaqus will bias the mesh toward the
            location of the maximum solution value in the region. The higher the value, the more the
            mesh will bias towards the location of the maximum solution value. You use the
            **meshBias** argument when **sizingMethod** = MINIMUM_MAXIMUM. The default value is 0.0.
        minElementSize
            A Float specifying the minimum size of any single element. The default value is 0.0.
        maxElementSize
            A Float specifying the maximum size of any single element. The default value is 0.0.
        outputFrequency
            A SymbolicConstant specifying the frequency with which the error indicators are saved to
            the output database file (.odb). Possible values are LAST_INCREMENT and ALL_INCREMENTS.
            The default value is LAST_INCREMENT.
        specifyMinSize
            A Boolean specifying an indication of whether to use a user-supplied minimum element
            size or to calculate a characteristic minimum element size. The default value is OFF.
        specifyMaxSize
            A Boolean specifying an indication of whether to use a user-supplied maximum element
            size or to calculate a characteristic maximum element size. The default value is ON.
        coarseningFactor
            A SymbolicConstant or an Int specifying an indication of the upper limit on the element
            growth from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT and
            NOT_ALLOWED. The default value is DEFAULT_LIMIT.
        refinementFactor
            A SymbolicConstant or an Int specifying an indication of the upper limit on element
            shrinkage from one remeshing iteration to the next. Possible values are DEFAULT_LIMIT
            and NOT_ALLOWED. The default value is DEFAULT_LIMIT.
        elementCountLimit
            None or an Int specifying an approximate limit on the number of elements that will be
            created during remeshing. Use None to indicate there is not upper limit. The default
            value is None.

        Returns
        -------
        RemeshingRule
            A :py:class:`~abaqus.Adaptivity.RemeshingRule.RemeshingRule` object
        """
        self.remeshingRules[name] = remeshingRule = RemeshingRule(
            name,
            stepName,
            variables,
            description,
            region,
            sizingMethod,
            errorTarget,
            maxSolutionErrorTarget,
            minSolutionErrorTarget,
            meshBias,
            minElementSize,
            maxElementSize,
            outputFrequency,
            specifyMinSize,
            specifyMaxSize,
            coarseningFactor,
            refinementFactor,
            elementCountLimit,
        )
        return remeshingRule

    @abaqus_method_doc
    def VelocityAdaptiveMeshConstraint(
        self,
        name: str,
        createStepName: str,
        region: Region,
        v1: typing.Union[SymbolicConstant, float] = UNSET,
        v2: typing.Union[SymbolicConstant, float] = UNSET,
        v3: typing.Union[SymbolicConstant, float] = UNSET,
        vr1: typing.Union[SymbolicConstant, float] = UNSET,
        vr2: typing.Union[SymbolicConstant, float] = UNSET,
        vr3: typing.Union[SymbolicConstant, float] = UNSET,
        amplitude: str = UNSET,
        localCsys: str = None,
        motionType: SymbolicConstant = INDEPENDENT,
    ) -> VelocityAdaptiveMeshConstraint:
        """This method creates a VelocityAdaptiveMeshConstraint object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].AdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the adaptive mesh constraint repository key.
        createStepName
            A String specifying the name of the step in which the adaptive mesh constraint is
            created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the adaptive mesh constraint is applied.
        v1
            A Float or a SymbolicConstant specifying the velocity component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET.Note:Although **v1**, **v2**, **v3**, **vr1**, **vr2**, and **vr3** are optional arguments, at
            least one of them must be specified.
        v2
            A Float or a SymbolicConstant specifying the velocity component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        v3
            A Float or a SymbolicConstant specifying the velocity component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        vr1
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr2
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr3
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the adaptive mesh constraint has no amplitude reference. The
            default value is UNSET. You should provide the **amplitude** argument only if it is valid
            for the specified step.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the adaptive mesh
            constraint's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        motionType
            A SymbolicConstant specifying the mesh motion in relation to the underlying material.
            Possible values are INDEPENDENT, FOLLOW and USER_DEFINED. The default value is
            INDEPENDENT.

        Returns
        -------
        VelocityAdaptiveMeshConstraint
            A :py:class:`~abaqus.Adaptivity.VelocityAdaptiveMeshConstraint.VelocityAdaptiveMeshConstraint` object
        """
        self.adaptiveMeshConstraints[
            name
        ] = adaptiveMeshConstraint = VelocityAdaptiveMeshConstraint(
            name,
            createStepName,
            region,
            v1,
            v2,
            v3,
            vr1,
            vr2,
            vr3,
            amplitude,
            localCsys,
            motionType,
        )
        return adaptiveMeshConstraint



import typing

from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class SolverControl:
    """The SolverControl object is used to provide additional optional solver controls.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import step
            mdb.models[name].steps[name].solverControl
    """

    @abaqus_method_doc
    def setValues(
        self,
        allowPropagation: Boolean = ON,
        resetDefaultValues: Boolean = OFF,
        relativeTolerance: typing.Union[SymbolicConstant, float] = DEFAULT,
        maxIterations: SymbolicConstant = DEFAULT,
        fillInLevel: SymbolicConstant = DEFAULT,
    ):
        """This method modifies the SolverControl object.

        Parameters
        ----------
        allowPropagation
            A Boolean specifying whether to allow all solver control values to propagate from a
            previous step. Setting this argument to ON automatically sets **resetDefaultValues** to
            OFF. The default value is ON.
        resetDefaultValues
            A Boolean specifying whether to use all default solver control values. Setting this
            argument to ON automatically sets **allowPropagation**to OFF. The default value is OFF.
        relativeTolerance
            The SymbolicConstant DEFAULT or a Float specifying the relative tolerance for
            convergence of the domain decomposition iterative solver. The default value is DEFAULT.
        maxIterations
            The SymbolicConstant DEFAULT or an Int specifying the maximum number of linear solver
            iterations. The default value is DEFAULT.
        fillInLevel
            The SymbolicConstant DEFAULT or an Int specifying the incomplete LU factorization
            fill-in level (for geostatic and soil analysis only). The default value is DEFAULT.

        Raises
        ------
        RangeError
        """
        ...



from .BoundaryConditionState import BoundaryConditionState
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc


@abaqus_class_doc
class AccelerationBaseMotionBCState(BoundaryConditionState):
    """The AccelerationBaseMotionBCState object stores the propagating data for a velocity base
    motion boundary condition in a step. One instance of this object is created internally
    by the AccelerationBaseMotionBC object for each step. The instance is also deleted
    internally by the AccelerationBaseMotionBC object.
    The AccelerationBaseMotionBCState object has no constructor or methods.
    The AccelerationBaseMotionBCState object is derived from the BoundaryConditionState
    object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - BASE MOTION
    """

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #: NOT_YET_ACTIVE
    #: CREATED
    #: PROPAGATED
    #: MODIFIED
    #: DEACTIVATED
    #: NO_LONGER_ACTIVE
    #: TYPE_NOT_APPLICABLE
    #: INSTANCE_NOT_APPLICABLE
    #: PROPAGATED_FROM_BASE_STATE
    #: MODIFIED_FROM_BASE_STATE
    #: DEACTIVATED_FROM_BASE_STATE
    #: BUILT_INTO_MODES
    status: SymbolicConstant = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class FeatureOptions:
    """The FeatureOptions object stores the options that control the behavior of feature
    regeneration for all features in a model.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import part
            mdb.models[name].featureOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        checkSelfIntersection: Boolean = ON,
        autoCaching: Boolean = ON,
        maxCachedStates: int = 5,
    ):
        """This method modifies the FeatureOptions object for the specified model.

        Parameters
        ----------
        checkSelfIntersection
            A Boolean specifying whether Abaqus/CAE should perform self-intersection checks while
            regenerating features. The default value is ON.
        autoCaching
            A Boolean specifying whether geometric states should be automatically cached. The
            default value is ON.
        maxCachedStates
            An Int specifying the maximum number of caches to be stored with each part or with the
            assembly. The default value is 5.
        """
        ...



import typing

from .AnalysisStep import AnalysisStep
from ..Adaptivity.AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..Adaptivity.AdaptiveMeshDomain import AdaptiveMeshDomain
from ..BoundaryCondition.BoundaryConditionState import BoundaryConditionState
from ..Load.LoadCase import LoadCase
from ..Load.LoadState import LoadState
from ..PredefinedField.PredefinedFieldState import PredefinedFieldState
from ..StepMiscellaneous.Control import Control
from ..StepMiscellaneous.SolverControl import SolverControl
from ..StepOutput.DiagnosticPrint import DiagnosticPrint
from ..StepOutput.FieldOutputRequestState import FieldOutputRequestState
from ..StepOutput.HistoryOutputRequestState import HistoryOutputRequestState
from ..StepOutput.Monitor import Monitor
from ..StepOutput.Restart import Restart
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc


@abaqus_class_doc
class InitialStep(AnalysisStep):
    """The InitialStep object is a placeholder that you cannot create, delete, or modify. The
    InitialStep object exists in every model by default and is used to allow the preexisting
    boundary conditions and interactions to be defined in the model.
    The InitialStep object is derived from the AnalysisStep object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import step
            mdb.models[name].steps[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A SymbolicConstant specifying whether the step has an explicit procedure type
    #: (*procedureType*=ANNEAL, DYNAMIC_EXPLICIT, or DYNAMIC_TEMP_DISPLACEMENT).
    explicit: SymbolicConstant = None

    #: A Boolean specifying whether the step has a perturbation procedure type.
    perturbation: Boolean = OFF

    #: A Boolean specifying whether the step has a mechanical procedure type.
    nonmechanical: Boolean = OFF

    #: A SymbolicConstant specifying the Abaqus procedure. Possible values are:
    #: 
    #: - ANNEAL
    #: - BUCKLE
    #: - COMPLEX_FREQUENCY
    #: - COUPLED_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRIC
    #: - DIRECT_CYCLIC
    #: - DYNAMIC_IMPLICIT
    #: - DYNAMIC_EXPLICIT
    #: - DYNAMIC_SUBSPACE
    #: - DYNAMIC_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRICAL_STRUCTURAL
    #: - FREQUENCY
    #: - GEOSTATIC
    #: - HEAT_TRANSFER
    #: - MASS_DIFFUSION
    #: - MODAL_DYNAMICS
    #: - RANDOM_RESPONSE
    #: - RESPONSE_SPECTRUM
    #: - SOILS
    #: - STATIC_GENERAL
    #: - STATIC_LINEAR_PERTURBATION
    #: - STATIC_RIKS
    #: - STEADY_STATE_DIRECT
    #: - STEADY_STATE_MODAL
    #: - STEADY_STATE_SUBSPACE
    #: - VISCO
    procedureType: SymbolicConstant = None

    #: A Boolean specifying whether the step is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A repository of FieldOutputRequestState objects.
    fieldOutputRequestState: typing.Dict[str, FieldOutputRequestState] = {}

    #: A repository of HistoryOutputRequestState objects.
    historyOutputRequestState: typing.Dict[str, HistoryOutputRequestState] = {}

    #: A :py:class:`~abaqus.StepOutput.DiagnosticPrint.DiagnosticPrint` object.
    diagnosticPrint: DiagnosticPrint = DiagnosticPrint()

    #: A :py:class:`~abaqus.StepOutput.Monitor.Monitor` object.
    monitor: Monitor = None

    #: A :py:class:`~abaqus.StepOutput.Restart.Restart` object.
    restart: Restart = Restart()

    #: A repository of AdaptiveMeshConstraintState objects.
    adaptiveMeshConstraintStates: typing.Dict[str, AdaptiveMeshConstraintState] = {}

    #: A repository of AdaptiveMeshDomain objects.
    adaptiveMeshDomains: typing.Dict[str, AdaptiveMeshDomain] = {}

    #: A :py:class:`~abaqus.StepMiscellaneous.Control.Control` object.
    control: Control = Control()

    #: A :py:class:`~abaqus.StepMiscellaneous.SolverControl.SolverControl` object.
    solverControl: SolverControl = SolverControl()

    #: A repository of BoundaryConditionState objects.
    boundaryConditionStates: typing.Dict[str, BoundaryConditionState] = {}

    #: A repository of InteractionState objects.
    interactionStates: int = None

    #: A repository of LoadState objects.
    loadStates: typing.Dict[str, LoadState] = {}

    #: A repository of LoadCase objects.
    loadCases: typing.Dict[str, LoadCase] = {}

    #: A repository of PredefinedFieldState objects.
    predefinedFieldStates: typing.Dict[str, PredefinedFieldState] = {}



from .ConstrainedSketchGeometry import ConstrainedSketchGeometry
from ...UtilityAndView.abaqusConstants import *
from ..._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class Spline(ConstrainedSketchGeometry):
    @abaqus_method_doc
    def __init__(self, points: tuple, constrainPoints: Boolean = True):
        """This method creates a spline curve running through a sequence of points.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].sketches[name].Spline

        Parameters
        ----------
        points
            A sequence of pairs of Floats specifying the points through which the spline passes.
        constrainPoints
            A Boolean that determines whether the points given are to constrained to always remain
            on the __init__. The default is True. For a large sequence of **points**, significant
            performance gains may be achieved by setting the value to False.

        Returns
        -------
        ConstrainedSketchGeometry
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object (None if the spline cannot be created).

        """
        ...



from .JobFromInputFile import JobFromInputFile
from .ModelJob import ModelJob
from .OptimizationProcess import OptimizationProcess
from ..Mdb.MdbBase import MdbBase
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class JobMdb(MdbBase):
    """The Mdb object is the high-level Abaqus model database. A model database stores models
    and analysis controls.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            mdb
    """

    @abaqus_method_doc
    def Job(
        self,
        name: str,
        model: str,
        description: str = "",
        type: SymbolicConstant = ANALYSIS,
        queue: str = "",
        waitHours: int = 0,
        waitMinutes: int = 0,
        atTime: str = "",
        echoPrint: Boolean = OFF,
        contactPrint: Boolean = OFF,
        modelPrint: Boolean = OFF,
        historyPrint: Boolean = OFF,
        scratch: str = "",
        userSubroutine: str = "",
        numCpus: int = 1,
        memory: int = 90,
        memoryUnits: SymbolicConstant = PERCENTAGE,
        explicitPrecision: SymbolicConstant = SINGLE,
        nodalOutputPrecision: SymbolicConstant = SINGLE,
        parallelizationMethodExplicit: SymbolicConstant = DOMAIN,
        numDomains: int = 1,
        activateLoadBalancing: Boolean = OFF,
        multiprocessingMode: SymbolicConstant = DEFAULT,
        licenseType: SymbolicConstant = DEFAULT,
        *args,
        **kwargs
    ) -> ModelJob:
        """This method creates an analysis job using a model on a model database (MDB) for the
        model definition.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.Job

        Parameters
        ----------
        name
            A String specifying the name of the new job. The name must be a valid Abaqus/CAE object
            name.
        model
            A String specifying the name of the model to be analyzed or a Model object specifying
            the model to be analyzed.
        description
            A String specifying a description of the job.
        type
            A SymbolicConstant specifying the type of job. Possible values are ANALYSIS,
            SYNTAXCHECK, RECOVER, and RESTART. The default value is ANALYSIS.If the object has the
            type JobFromInputFile, **type** = RESTART is not available.
        queue
            A String specifying the name of the queue to which to submit the job. The default value
            is an empty string.Note:You can use the **queue** argument when creating a Job object on a
            Windows workstation; however, remote queues are available only on Linux platforms.
        waitHours
            An Int specifying the number of hours to wait before submitting the job. This argument
            is ignored if **queue** is set. The default value is 0.This argument works in conjunction
            with **waitMinutes**. **waitHours** and **atTime** are mutually exclusive.
        waitMinutes
            An Int specifying the number of minutes to wait before submitting the job. This argument
            is ignored if **queue** is set. The default value is 0.This argument works in conjunction
            with **waitHours**. **waitMinutes** and **atTime** are mutually exclusive.
        atTime
            A String specifying the time at which to submit the job. If **queue** is empty, the string
            syntax must be valid for the Linux `at` command. If **queue** is set, the syntax must be
            valid according to the system administrator. The default value is an empty
            string.Note:You can use the **atTime** argument when creating a Job object on a Windows
            workstation; however, the `at` command is available only on Linux platforms.
        echoPrint
            A Boolean specifying whether an echo of the input data is printed. The default value is
            OFF.
        contactPrint
            A Boolean specifying whether contact constraint data are printed. The default value is
            OFF.
        modelPrint
            A Boolean specifying whether model definition data are printed. The default value is
            OFF.
        historyPrint
            A Boolean specifying whether history data are printed. The default value is OFF.
        scratch
            A String specifying the location of the scratch directory. The default value is an empty
            string.
        userSubroutine
            A String specifying the file containing the user's subroutine definitions. The default
            value is an empty string.
        numCpus
            An Int specifying the number of CPUs to use for this analysis if parallel processing is
            available. Possible values are **numCpus** >> 0. The default value is 1.
        memory
            An Int specifying the amount of memory available to Abaqus analysis. The value should be
            expressed in the units supplied in **memoryUnits**. The default value is 90.
        memoryUnits
            A SymbolicConstant specifying the units for the amount of memory used in an Abaqus
            analysis. Possible values are PERCENTAGE, MEGA_BYTES, and GIGA_BYTES. The default value
            is PERCENTAGE.
        explicitPrecision
            A SymbolicConstant specifying whether to use the double precision version of
            Abaqus/Explicit. Possible values are SINGLE, FORCE_SINGLE, DOUBLE,
            DOUBLE_CONSTRAINT_ONLY, and DOUBLE_PLUS_PACK. The default value is SINGLE.
        nodalOutputPrecision
            A SymbolicConstant specifying the precision of the nodal output written to the output
            database. Possible values are SINGLE and FULL. The default value is SINGLE.
        parallelizationMethodExplicit
            A SymbolicConstant specifying the parallelization method for Abaqus/Explicit. This value
            is ignored for Abaqus/Standard. Possible values are DOMAIN and LOOP. The default value
            is DOMAIN.
        numDomains
            An Int specifying the number of domains for parallel execution in Abaqus/Explicit. When
            **parallelizationMethodExplicit** = DOMAIN, **numDomains** must be a multiple of **numCpus**.
            The default value is 1.
        activateLoadBalancing
            A Boolean specifying whether to activate dyanmic load balancing for jobs running on
            multiple processors with multiple domains in Abaqus/Explicit. The default value is OFF.
        multiprocessingMode
            A SymbolicConstant specifying whether an analysis is decomposed into threads or into
            multiple processes that communicate through a message passing interface (MPI). Possible
            values are DEFAULT, THREADS, and MPI. The default value is DEFAULT.
        licenseType
            A SymbolicConstant specifying the type of license type being used in the case of the
            DSLS SimUnit license model. Possible values are DEFAULT, TOKEN, and CREDIT. The default
            value is DEFAULT.If the license model is not the DSLS SimUnit, the licenseType is not
            available.

            .. versionchanged:: 2022
                The `licenseType` argument was added.

        Returns
        -------
        ModelJob
            A :py:class:`~abaqus.Job.ModelJob.ModelJob` object.

        Raises
        ------
        AbaqusException
        """
        self.jobs[name] = job = ModelJob(
            name,
            model,
            description,
            type,
            queue,
            waitHours,
            waitMinutes,
            atTime,
            echoPrint,
            contactPrint,
            modelPrint,
            historyPrint,
            scratch,
            userSubroutine,
            numCpus,
            memory,
            memoryUnits,
            explicitPrecision,
            nodalOutputPrecision,
            parallelizationMethodExplicit,
            numDomains,
            activateLoadBalancing,
            multiprocessingMode,
            licenseType,
            *args,
            **kwargs
        )
        return job

    @abaqus_method_doc
    def JobFromInputFile(
        self,
        name: str,
        inputFileName: str,
        type: SymbolicConstant = ANALYSIS,
        queue: str = "",
        waitHours: int = 0,
        waitMinutes: int = 0,
        atTime: str = "",
        scratch: str = "",
        userSubroutine: str = "",
        numCpus: int = 1,
        memory: int = 90,
        memoryUnits: SymbolicConstant = PERCENTAGE,
        explicitPrecision: SymbolicConstant = SINGLE,
        nodalOutputPrecision: SymbolicConstant = SINGLE,
        parallelizationMethodExplicit: SymbolicConstant = DOMAIN,
        numDomains: int = 1,
        activateLoadBalancing: Boolean = OFF,
        multiprocessingMode: SymbolicConstant = DEFAULT,
        licenseType: SymbolicConstant = DEFAULT,
    ) -> JobFromInputFile:
        """This method creates an analysis job using an input file for the model definition.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.JobFromInputFile

        Parameters
        ----------
        name
            A String specifying the name of the new job. The name must be a valid Abaqus/CAE object
            name.
        inputFileName
            A String specifying the input file to read. Possible values are any valid file name. If
            the .inp extension is not included in the value of the argument, the system will append
            it for the user.
        type
            A SymbolicConstant specifying the type of job. Possible values are ANALYSIS,
            SYNTAXCHECK, and RECOVER. The default value is ANALYSIS.For theJobFromInputFile object,
            **type** = RESTART is not currently supported.
        queue
            A String specifying the name of the queue to which to submit the job. The default value
            is an empty string.Note: You can use the **queue** argument when creating a Job object on
            a Windows workstation; however, remote queues are available only on Linux platforms.
        waitHours
            An Int specifying the number of hours to wait before submitting the job. This argument
            is ignored if **queue** is set. The default value is 0.This argument works in conjunction
            with **waitMinutes**. **waitHours** and **atTime** are mutually exclusive.
        waitMinutes
            An Int specifying the number of minutes to wait before submitting the job. This argument
            is ignored if **queue** is set. The default value is 0.This argument works in conjunction
            with **waitHours**. **waitMinutes** and **atTime** are mutually exclusive.
        atTime
            A String specifying the time at which to submit the job. If **queue** is empty, the string
            syntax must be valid for the Linux `at` command. If **queue** is set, the syntax must be
            valid according to the system administrator. The default value is an empty string.Note:
            You can use the **atTime** argument when creating a Job object on a Windows workstation;
            however, the `at` command is available only on Linux platforms.
        scratch
            A String specifying the location of the scratch directory. The default value is an empty
            string.
        userSubroutine
            A String specifying the file containing the user's subroutine definitions. The default
            value is an empty string.
        numCpus
            An Int specifying the number of CPUs to use for this analysis if parallel processing is
            available. Possible values are **numCpus** >> 0. The default value is 1.
        memory
            An Int specifying the amount of memory available to Abaqus analysis. The value should be
            expressed in the units supplied in **memoryUnits**. The default value is 90.
        memoryUnits
            A SymbolicConstant specifying the units for the amount of memory used in an Abaqus
            analysis. Possible values are PERCENTAGE, MEGA_BYTES, and GIGA_BYTES. The default value
            is PERCENTAGE.
        explicitPrecision
            A SymbolicConstant specifying whether to use the double precision version of
            Abaqus/Explicit. Possible values are SINGLE, FORCE_SINGLE, DOUBLE,
            DOUBLE_CONSTRAINT_ONLY, and DOUBLE_PLUS_PACK. The default value is SINGLE.
        nodalOutputPrecision
            A SymbolicConstant specifying the precision of the nodal output written to the output
            database. Possible values are SINGLE and FULL. The default value is SINGLE.
        parallelizationMethodExplicit
            A SymbolicConstant specifying the parallelization method for Abaqus/Explicit. This value
            is ignored for Abaqus/Standard. Possible values are DOMAIN and LOOP. The default value
            is DOMAIN.
        numDomains
            An Int specifying the number of domains for parallel execution in Abaqus/Explicit. When
            **parallelizationMethodExplicit** = DOMAIN, **numDomains** must be a multiple of **numCpus**.
            The default value is 1.
        activateLoadBalancing
            A Boolean specifying whether to activate dyanmic load balancing for jobs running on
            multiple processors with multiple domains in Abaqus/Explicit. The default value is OFF.
        multiprocessingMode
            A SymbolicConstant specifying whether an analysis is decomposed into threads or into
            multiple processes that communicate through a message
        licenseType
            A SymbolicConstant specifying the type of license type being used in the case of the
            DSLS SimUnit license model. Possible values are DEFAULT, TOKEN, and CREDIT. The default
            value is DEFAULT.If the license model is not the DSLS SimUnit, the licenseType is not
            available.
        """

        self.jobs[name] = jobFromInputFile = JobFromInputFile(
            name,
            inputFileName,
            type,
            queue,
            waitHours,
            waitMinutes,
            atTime,
            scratch,
            userSubroutine,
            numCpus,
            memory,
            memoryUnits,
            explicitPrecision,
            nodalOutputPrecision,
            parallelizationMethodExplicit,
            numDomains,
            activateLoadBalancing,
            multiprocessingMode,
            licenseType,
        )
        return jobFromInputFile

    @abaqus_method_doc
    def OptimizationProcess(
        self,
        name: str,
        model: str,
        task: str,
        prototypeJob: str,
        description: str = "",
        maxDesignCycle: int = 15,
        dataSaveFrequency: str = OPT_DATASAVE_SPECIFY_CYCLE,
        saveInitial: Boolean = True,
        saveFirst: Boolean = True,
        saveLast: Boolean = True,
        saveEvery: int = None,
    ) -> OptimizationProcess:
        """This method creates an OptimizationProcess object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.OptimizationProcess

        Parameters
        ----------
        name
            A String specifying name of the optimization process.
        model
            A String specifying name of the model to be used for the optimization process.
        task
            A String specifying name of the optimization task to be used for the optimization
            process.
        prototypeJob
            A String specifying name of the job to be used as the prototype for all analysis jobs
            run by the optimization process.
        description
            A String specifying a description of the optimization process.
        maxDesignCycle
            An Int specifying the maximum number of allowed design cycles for the optimization
            process. The default value is 15.
        dataSaveFrequency
            An Enum specifying whether Abaqus should save every iteration file in the optimization
            process or a selection of iteration files saved at a user-specified frequency. If you
            set **dataSaveFrequency** = OPT_DATASAVE_EVERY_CYCLE, Abaqus saves every iteration file; if
            you set **dataSaveFrequency** = OPT_DATASAVE_SPECIFY_CYCLE, Abaqus saves iteration files
            according to the frequency defined by the **saveEvery** parameter. The default value is
            OPT_DATASAVE_SPECIFY_CYCLE.
        saveInitial
            A Boolean specifying whether the initial cycle should be saved when **dataSaveFrequency**
            is OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveFirst
            A Boolean specifying whether the first cycle should be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveLast
            A Boolean specifying whether the last cycle should be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. The default value is True.
        saveEvery
            An Int specifying every nth cycle iterations to be saved when **dataSaveFrequency** is
            OPT_DATASAVE_SPECIFY_CYCLE. Abaqus saves file iterations for every nth iteration after
            iteration 1; if you set **saveEvery** = 3, Abaqus saves file iterations for cycles 1, 4, 7,
            and so on. The default value is None.

        Returns
        -------
        OptimizationProcess
            An :py:class:`~abaqus.Job.OptimizationProcess.OptimizationProcess` object.

        Raises
        ------
        AbaqusException
        """
        self.optimizationProcesses[name] = optimizationProcess = OptimizationProcess(
            name,
            model,
            task,
            prototypeJob,
            description,
            maxDesignCycle,
            dataSaveFrequency,
            saveInitial,
            saveFirst,
            saveLast,
            saveEvery,
        )
        return optimizationProcess



import typing

from .ElementProgressiveActivation import ElementProgressiveActivation
from ..Assembly.AssemblyBase import AssemblyBase
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class TableCollectionAssembly(AssemblyBase):
    """An :py:class:`~abaqus.Assembly.Assembly.Assembly` object is a container for instances of parts. The Assembly object has no
    constructor command. Abaqus creates the **rootAssembly** member when a Model object is
    created.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import assembly
            mdb.models[name].rootAssembly

    .. versionadded:: 2020
        The `TableCollectionAssembly` class was added.
    """

    elementProgressiveActivations: typing.Dict[str, ElementProgressiveActivation] = {}

    @abaqus_method_doc
    def ElementProgressiveActivation(
        self,
        name: str,
        elset: Region,
        deformation: Boolean = OFF,
        freeSurfaceType: SymbolicConstant = NONE,
    ) -> ElementProgressiveActivation:
        """This method creates an ElementProgressiveActivation object and places it in the
        elementProgressiveActivation repository.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].rootAssembly.ElementProgressiveActivation

        Parameters
        ----------
        name
            A String specifying the key of the repository.
        elset
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region containing the elements that will be activated
            during the analysis.
        deformation
            A Boolean value specifying whether the elements that have not yet been activated will
            follow the deformations of the active elements. Set **deformation** = ON when the
            deformation of the active elements is excessive. The default value is OFF.
        freeSurfaceType
            A SymbolicConstant specifying the exposed areas of the element facets that are active
            for convection or radiation boundary conditions to be applied. Possible values are NONE
            and FACET. If **freeSurfaceType** = FACET, user subroutine UEPACTIVATIONFACET will be called
            at the start of the increment for each element. If **freeSurfaceType** = NONE, all the
            exposed areas of the element facets are considered. The default value is NONE.

        Returns
        -------
        ElementProgressiveActivation
            An :py:class:`~abaqus.TableCollection.ElementProgressiveActivation.ElementProgressiveActivation` object.

        Raises
        ------
        AbaqusException
            If the region does not contain only elements.
        """
        self.elementProgressiveActivations[
            name
        ] = elementProgressiveActivation = ElementProgressiveActivation(
            name, elset, deformation, freeSurfaceType
        )
        return elementProgressiveActivation



from ..UtilityAndView.abaqusConstants import *
from .._decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class DisplayGroupInstanceRepository:
    """The DisplayGroupInstanceRepository object stores DisplayGroupInstance objects. In
    addition to all the standard Python repository methods, the DisplayGroupInstance
    repository defines additional methods as described below.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import visualization
            session.viewports[name].layers[name].odbDisplay.displayGroupInstances
            session.viewports[name].odbDisplay.displayGroupInstances
    """

    @abaqus_method_doc
    def syncOptions(self, name: str, updateInstances: Boolean = ON):
        """This method synchronizes the display options stored on the OdbDisplay object with the
        display options stored on the DisplayGroupInstance object.

        Parameters
        ----------
        name
            A String specifying the repository key.
        updateInstances
            A Boolean specifying whether to synchronize the display options on all the
            DisplayGroupInstance objects stored in the DisplayGroupInstanceRepository for which
            **lockOptions** is OFF. The default value of **updateInstances** is ON.
        """
        ...
